unit uCaves;

interface

procedure DrawCarpet;
procedure DrawBackWall;
procedure DrawBars;
procedure DrawBridge;
procedure DrawDart;
procedure DrawDebris;
procedure DrawEmpty;
procedure DrawDoorLeft;
procedure DrawDoorMovable;
procedure DrawDoorRight;
procedure DrawFloor;
procedure DrawGate;
procedure DrawGateMovable;
procedure DrawLavaBubble;
procedure DrawLavaLeft;
procedure DrawLavaRight;
procedure DrawLoose;
procedure DrawPillarBottom;
procedure DrawPillarTop;
procedure DrawPost;
procedure DrawPressure;
procedure DrawSpikes;
procedure DrawTorch;
procedure DrawTorchDebris;
procedure DrawWall;
procedure CarpetRoomTemplateIn;
procedure CarpetRoomTemplateOut;
procedure InitCarpet;
procedure DeinitCarpet;
function CheckCarpetImg(index: word): pointer;
function GetCarpetShape(index: word): pointer;
function OffsetBrgFloorY(offset: smallint; col: smallint): smallint;
procedure RestoreBrgFloorY(idx: smallint; prev: smallint);
procedure DrawFrontWall;
procedure DrawTheDebris;



implementation

uses uMacFile, uColorTables, uPop2Types, uGlobals, uResources, uShapes, uImgTables, uSwapper,
uRect, uMacTypes, uPieces, uRoomUtil, Math;

const
dartitems : array [0..6] of word = ($5B, $5D, $5C, $5B, $5B, $5B, $5B);
loosetops: array [0..12] of smallint = (0,1,0,1,1,0,0,0,1,0,1,0,0);
brgoffsets: array[1..10] of smallint = (2,4,6,7,7,7,6,3,7,8);

var

frontinfos: array[0..14] of array[0..2] of smallint =
(
(95,  0,  -86),
(96,  0, -$4A),
(97,  0, -$43),
(98,  0, -1),
(99,  0, -1),
(100, 0, 0),
(101, 0, -$39),
(102, 0, 0),
(103, 0, 0),
(104, 0, -$66),
(105, 0, 0),
(106, 0, -$43),
(107, 0, -$41),
(108, 0, -$32),
(108, 0, -$64)
);

type
PCarpet = ^TCarpet;
TCarpet = record
  param1: word;
  param2: word;
  param3: word;
end;

var
carpet: PCarpet;

procedure DrawCarpet;
var
cshape: pbyte;
begin
  if carpet = nil then exit;
  if carpet^.param1 <> 0 then exit;
  if NextBFItem^.layer <> 0 then exit;
  if not drawingfull then exit;
  cshape := GetCarpetShape(9);
  NextBFItem^.Top := $16B - swap2(pword(integer(cshape)+6)^);
  NextBFItem^.Left := $95;
  NextBFItem^.ItemID := 25311;
  MakeShapeRect(cshape, NextBFItem^.Top, NextBFItem^.Left, NextBFItem^.ClippedRect);
  if SectRect(Backbuffer.Rect, NextBFItem^.ClippedRect, NextBFItem^.ClippedRect) then
  begin
    inc(BackImgCount);
    NextBFItem := nil;
  end;
end;

procedure DrawBackWall;
var
mask1, mask2: word;
altview: PBFImgENtry;
rect: TMacRect;
willdraw: boolean;
prevfore: word;
begin
mask1 := (NextBFItem^.PartBack.Back2 and $F00) shr 8;
mask2 := (NextBFItem^.PartBack.Back2 and $1F);
if mask2 = 0 then exit;
if not drawingfull then
begin
  altview := GetBackTableImgRow(mask2, NextBFItem^.PartCol -1, NextBFItem.PartRow);
  if (altview <> nil) and SectRect(drawingrect, altview^.ClippedRect, rect) then
  begin
      UnionRect(drawingrect, altview^.ClippedRect, altview.ClippedRect);
      willdraw := false;
  end
  else
  begin
    altview := nil;
    willdraw := true;
  end;
end
else
begin
  willdraw := false;
  altview := nil;
end;
if (altview = nil) and (willdraw or (mask1 = 0)) and (mask2 <> 0) then
begin
  prevfore := NextBFItem^.PartFore;
  NextBFItem^.PartFore := 0;
  NextBFItem^.ItemID := mask2;
  DrawPiecePos(0, mask1);
  NextBFItem^.Transparent := 0;
  if AddNextBFItemToTable then
    ResetNextBFItemWithPrevInfo(0);
  NextBFItem^.PartFore := prevfore;
  NextBFItem^.Transparent := $10;
end;

end;

procedure DrawBars;
var
oldrect: TMacRect;
begin
  if (NextBFItem^.Layer <> 11) or (NextBFItem^.partcol <> drawingcol) then exit;
  oldrect := drawingrect;
  drawingrect := backbuffer.Rect;
  DrawCustomRoomImg((NextBFItem^.PartBack.Back1 and 7) + 1, 11);
  drawingrect := oldrect;
end;

procedure DrawBridge;
var
backmasked: word;
prevoffset: smallint;
begin
if NextBFItem^.layer in [2,5] then
begin
  backmasked := (NextBFItem^.PartBack.Back1 and $F) + 3;
  prevoffset := OffsetBrgFloorY(backmasked, NextBFItem.PartCol);
  DrawCustomRoomImgAtInfo(backmasked);
  RestoreBrgFloorY(backmasked, prevoffset);
end;
end;

procedure DrawDart;
var
prevback: word;
layer: word;
backmasked: word;
begin
layer := NextBFItem^.layer;
prevback := NextBFItem^.PartBack.Back1;
NextBFItem^.PartBack.Back1 := (NextBFItem^.PartBack.Back1 and $FFFC) or 1;
DrawFloor;
if NextBFItem = nil then
  ResetNextBFItemWithPrevInfo(layer);
NextBFItem^.PartBack.Back1 := prevback;
if layer <> 1 then exit;
backmasked := prevback and 7;
NextBFItem^.ItemID := dartitems[backmasked];
DrawPiecePos;
AddNextBFItemToTable;
end;

procedure DrawDebris;
var
layer: word;
begin
layer := NextBFItem^.Layer;
if (NextBFItem^.PartBack.back1 and $2000) <> 0 then
  DrawPressure
else
  DrawFloor;
if NextBFItem = nil then
  ResetNextBFItemWithPrevInfo(layer);
DrawTheDebris;
end;

procedure DrawEmpty;
begin
  case NextBFItem^.Layer of
  0: DrawBackWall;
  1: DrawFrontWall;
  5: begin
    if (NextBFItem^.partback.back1 and 7) >= 3 then
    begin
      NextBFItem^.PartBack.back1 := (NextBFItem^.PartBack.back1 and $F00) or $D;
      NextBFItem^.PartFore := CELL_LOOSE;
      DrawLoose;
    end;
  end;
  end;
end;

procedure DrawDoorLeft;
begin
if NextBFItem^.Layer = 0 then
begin
  DrawBackWall;
  NextBFItem^.ItemID := 64;
  DrawPiecePos;
  AddNextBFItemToTable;
end
else
begin
  NextBFItem^.PartBack.back1 := NextBFItem^.PartBack.back1 and $FFFC;
  DrawFloor;
end;

end;

procedure DrawDoorMovable;
var
oldclip: TMacRect;
offrect: TMacRect;
backmasked: word;
begin
  if NextBFItem^.Layer <> 0 then exit;
  oldclip := drawingrect;
  offrect := backbuffer.rect;
  offrect.top := NextBFItem^.PartRow * 120 + 1;
  if SectRect(offrect, drawingrect, drawingrect) then
  begin
    backmasked := min(NextBFItem^.partback.back1 and $FF, 79);
    NextBFItem^.ItemID := 66;
    NextBFITem.Left := currentpieces[57].items[0].left;
    NextBFItem.Top := -29 - (backmasked + 1);
    if AddNextBFItemToTable then
      ResetNextBFItemWithPrevInfo(0);
    //trucs qui concernent kid qui monte les marches.
    NextBFItem^.ItemID := 65;
    NextBFItem^.Left := currentpieces[57].items[1].Left;
    NextBFItem^.Top := currentpieces[57].items[1].Top;
    AddNextBFItemToTable;
  end;

  drawingrect := oldclip;
end;

procedure DrawDoorRight;
begin
  if NextBFItem^.layer = 0 then
  begin
    DrawBackWall;
    NextBFItem^.ItemID := 64;
    DrawPiecePos;
    if AddNextBFItemToTable then
      ResetNextBFItemWithPrevInfo(0);
    if currentroom = currentlevel.InitialRoom then
    begin
      NextBFItem^.ItemID := 67;
      DrawPiecePos(2);
      if AddNextBFItemToTable then
        ResetNextBFItemWithPrevInfo(0);
    end;
    DrawDoorMovable;
  end
  else
  begin
    NextBFItem^.PartBack.Back1 := (NextBFItem^.PartBack.Back1 and $FFFC) or 2;
    DrawFloor;
  end;

end;

procedure DrawFloor;
var
willdraw: boolean;
layer: word;
backmasked: word;
begin
willdraw := false;
layer := NextBFItem^.layer;
backmasked := NextBFItem^.PartBack.Back1 and 3;
case layer of
0: DrawBackWall;
1: begin
  if backmasked=3  then
   NextBFItem^.ItemID := 24
  else
  begin
    NextBFItem^.Transparent := 0;
    NextBFItem^.ItemID := 23;
  end;
  DrawPiecePos(-1, CELL_FLOOR);
  willdraw := true;
end;
2, 5: begin
willdraw := true;
NextBFItem^.ItemID := 25+backmasked;
if backmasked = 3 then
  NextBFItem^.Left := 9
else
  NextBFItem^.Left := 2;
NextBFItem^.Top := currentpieces[CELL_FLOOR].items[1].Top;
end;
end;

if not willdraw then exit;
  if AddNextBFItemToTable then
    ResetNextBFItemWithPrevInfo(layer);
  NextBFItem^.Transparent := $10;
  if layer = 1 then
    DrawFrontWall;


end;

procedure DrawGate;
var
prevback: word;
layer: word;
begin
layer := NextBFItem^.layer;
if layer = 0 then
begin
  NextBFItem^.ItemID := 61;
  NextBFItem^.Left := currentpieces[36].items[0].Left;
  NextBFItem^.Top := currentpieces[36].items[0].Top;
  if AddNextBFItemToTable then
    ResetNextBFItemWithPrevInfo(0);
end
else
begin
  prevback := NextBFItem^.PartBack.Back1;
  NextBFItem^.PartBack.Back1 := (prevback and $F0FC) or 2;
  DrawFloor;
  if NextBFItem = nil then
    ResetNextBFItemWithPrevInfo(layer);
  NextBFITem^.partback.Back1 := prevback;
end;
DrawGateMovable;
end;

procedure DrawGateMovable;
var
oldrect: TMacRect;
offrect: TMacRect;
idx: word;
toppos: smallint;
backmasked: word;
layer: word;
willdraw: boolean;
prevcol: smallint;
begin
  oldrect := drawingrect;
  offrect := backbuffer.rect;
  offrect.top := NextBFItem^.partrow * 120 - 31;
  if SectRect(offrect, drawingrect, drawingrect) then
  begin
    backmasked := NextBFItem^.PartBack.Back1 and $FF;
    idx := min($2E, backmasked shr 2);
    toppos := -7 - (idx shl 1);
    willdraw := false;
    layer := NextBFItem^.layer;
    case layer of
    1: begin
      NextBFItem^.ItemID := 62;
      NextBFItem^.Left := currentpieces[55].items[2].left;
      NextBFItem^.Top := toppos;
      if AddNextBFItemToTable then
        ResetNextBFItemWithPrevInfo(1);
      prevcol := drawingcol;
      drawingcol := NextBFItem^.PartCol;
      DrawFrontWall;
      drawingcol := prevcol;
      if NextBFItem = nil then
        ResetNextBFItemWithPrevInfo(1);
    end;
    2,5: begin
      NextBFItem^.ItemID := 63;
      NextBFItem^.Left := currentpieces[55].items[1].Left;
      NextBFItem^.Top := toppos-1;
      willdraw := true;
    end;
    end;
    if willdraw then
    begin
      if AddNextBFItemToTable then
        ResetNextBFItemWithPrevInfo(layer);
      if (layer <> 2) or (idx <> $2E) then
      begin
        if layer = 2 then
          drawingrect.bottom := toppos - 13;
      end;
      if not EmptyRect(drawingrect) then
      begin
        NextBFItem^.ItemID := 60;
        DrawPiecePos(5,CELL_DOOR);
        if AddNextBFItemToTable then
          ResetNextBFItemWithPrevInfo(layer);
      end;
    end;
  end;
  drawingrect := oldrect;

end;

procedure DrawLavaBubble;
begin
  //on mettra pas de bulles pour l'instant.
end;

procedure DrawLavaLeft;
var
layer: word;
begin
  case NextBFItem^.Layer of
  0: DrawBackWall;
  1: begin
    NextBFItem^.PartBack.Back1 := NextBFItem^.PartBack.Back1 and $F00;
    DrawFloor;
  end;
  2,5: begin
    if not drawingfull then
    begin
      layer := NextBFItem^.Layer;
      NextBFItem^.ItemID := 114;
      DrawPiecePos;
      if AddNextBFItemToTable then
        ResetNextBFItemWithPrevInfo(layer);
      DrawLavaBubble;
    end;
  end;
  end;
end;

procedure DrawLavaRight;
var
layer: word;
begin
  case NextBFItem^.Layer of
  0: DrawBackWall;
  1: begin
    NextBFItem^.PartBack.Back1 := NextBFItem^.PartBack.Back1 and $F00;
    DrawFloor;
  end;
  2, 5: begin
      layer := NextBFItem^.Layer;
      NextBFItem^.ItemID := 114;
      DrawPiecePos;
      if AddNextBFItemToTable then
        ResetNextBFItemWithPrevInfo(layer);
      DrawLavaBubble;
  end;
  end;
end;

procedure DrawLoose;
var
idx: smallint;
oldrect: TMacRect;
begin
idx := NextBFItem^.partback.back1 and $F;
if idx < 13 then
idx := loosetops[idx];
case NextBFItem^.layer of
0: DrawBackWall;
1: DrawFrontWall;
2: begin
  if idx <> 13 then
  begin
    currentpieces[11].items[1].top := loosetops[idx shl 1];
    NextBFItem^.ItemID := 57;
    DrawPiecePos;
    oldrect := drawingrect;
    //trucs ayant rapport avec kid
    inc(drawingrect.bottom);
    AddNextBFItemToTable;
    drawingrect := oldrect;
  end;
end;
5: begin
  NextBFItem^.ItemID := 56;
    DrawPiecePos(0);
  if AddNextBFItemToTable then
    ResetNextBFItemWithPrevInfo(5);
  if idx <> 13 then
  begin
    currentpieces[11].items[1].top := loosetops[idx shl 1];
    NextBFItem^.ItemID := 57;
    DrawPiecePos;
    oldrect := drawingrect;
    //trucs ayant rapport avec kid
    inc(drawingrect.bottom);
    AddNextBFItemToTable;
    drawingrect := oldrect;
  end;
end;
end;
end;

procedure DrawPillarBottom;
var
layer: word;
prevback: word;
begin
layer := NextBFItem^.layer;
prevback := NextBFItem^.partback.back1;
NextBFItem^.partback.Back1 := NextBFItem^.PartCol and 1;
DrawFloor;
if NextBFItem = nil then
  ResetNextBFItemWithPrevInfo(layer);
NextBFItem^.PartBack.Back1 := prevback;
case layer of
1: begin
  NextBFItem^.ItemID := 33;
  DrawPiecePos;
  AddNextBFItemToTable;
end;
2,5: begin
  NextBFItem^.ItemID := 34;
  DrawPiecePos;
  AddNextBFItemToTable;
end;
end;

end;

procedure DrawPillarTop;
begin
case NextBFItem^.Layer of
0: DrawBackWall;
1: begin
  NextBFItem^.ItemID := 35;
  DrawPiecePos;
  if AddNextBFItemToTable then
    ResetNextBFItemWithPrevInfo(1);
  DrawFrontWall;
end;
5: begin
  NextBFItem^.ItemID := 36;
  DrawPiecePos;
  AddNextBFItemToTable;
end;
end;
end;

procedure DrawPost;
var
prevback: word;
layer: word;
willdraw: boolean;
begin
prevback := NextBFItem^.PartBack.Back1;
layer := NextBFITem^.layer;
NextBFItem^.PartBack.Back1 := NextBFItem^.PartBack.Back1 or ((NextBFItem^.PartCol and 1) + 1);
DrawFloor;
if NextBFItem = nil then
  ResetNextBFItemWithPrevInfo(layer);
NextBFItem^.PartBack.Back1 := prevback;
willdraw := false;

case layer of
1: begin
  NextBFItem^.ItemID := 31;
  DrawPiecePos;
  willdraw := true;
end;
2,5: begin
  NextBFItem^.ItemID := 32;
  DrawPiecePos;
  willdraw := true;
end;
end;

if willdraw then
begin
  AddNextBFItemToTable;
  if layer = 1 then
  begin
    if NextBFItem = nil then
      ResetNextBFItemWithPrevInfo(1);
    DrawFrontWall;
  end;
end;


end;

procedure DrawPressure;
begin
case NextBFITem^.Layer of
0: DrawBackWall;
1: begin
  NextBFItem^.PartBack.Back1 := NextBFItem^.PartBack.Back1 and $CF00;
  if PieceAboveId(NextBFItem^.PartCol, NextBFItem^.partrow) = CELL_LOOSE then
    NextBFItem^.PartBack.Back1 := (NextBFItem^.PartBack.Back1 and $F0FF) or $A00;
  DrawFloor;
end;
2,5: begin
  if (NextBFItem^.PartBack.Back1 and $2000) <> 0 then
    NextBFItem^.ItemID := 55
  else
    NextBFItem^.ItemID := 54;
  DrawPiecePOs;
  AddNextBFItemToTable;
end;
end;
end;

procedure DrawSpikes;
begin
case NextBFItem^.layer of
0: DrawBackWall;
1: begin
  NextBFItem^.PartBack.Back1 := (NextBFItem^.PartBack.Back1 and $FFF0) or 4;
  DrawWall;
end;
5: begin
  if NextBFItem^.PartCol <> 9 then
  begin
    NextBFItem^.ItemID := 68;
    DrawPiecePos(0, CELL_WALL);
    AddNextBFItemToTable;
  end;
  
end;

end;
end;

procedure DrawTorch;
var
prevback: word;
frame: word;
begin
  prevback := NextBFItem^.PartBack.Back1;
  NextBFItem^.PartBack.Back1 := (prevback and $F00) or ((prevback and $E0) shr 5);
  case NextBFItem^.Layer of
  0:
  begin
    DrawFloor;
    if (prevback and $10) = 0 then
      frame := prevback and $F
    else
      frame := 9;
    if NextBFItem = nil then
      ResetNextBFItemWithPrevInfo(0);
    NextBFItem^.ItemID := frame + 81;
    DrawPiecePos(-1, CELL_TORCH);
    AddNextBFItemToTable;

    
  end;
  1,2,5: DrawFloor;
  end;
end;

procedure DrawTorchDebris;
var
layer: word;
begin
  layer := NextBFItem^.Layer;
  DrawTorch;
  if NextBFItem = nil then
    ResetNextBFItemWithPrevInfo(layer);
  DrawTheDebris;
end;

procedure DrawWall;
var
atright: word;
begin
  case NextBFItem^.Layer of
  1: begin
    NextBFItem^.ItemID := 37 + (NextBFItem^.PartBack.Back1 and $F);
    DrawPiecePos;
    if AddNextBFItemToTable then
      ResetNextBFItemWithPrevInfo(1);
    DrawFrontWall;
  end;
  5: begin
    if currentroom <> 0 then
    begin
      atright := PieceRightId(NextBFItem^.PartCol, NextBFItem^.PartRow);
    end;
    if (currentroom = 0) or (atright <> CELL_WALL) then
    begin
    NextBFItem^.ItemID :=  50+((NextBFItem^.PartBack.Back1 and $30) shr 4);
    DrawPiecePos(0, CELL_WALL);
    AddNextBFItemToTable;
    end;
  end;  
  end;
end;

procedure CarpetRoomTemplateIn;
var
curfile: PResource;
begin
  curfile := PopCurResFile;
  PopUseResFile(levelresources[currentlevel.leveltype]);
  PopSetPalette(25303, 32);
  PopSetPalette(3501, 64);
  InitCarpet;
  //AddSong(114);
  PopUseResFile(curfile);
end;

procedure CarpetRoomTemplateOut;
var
curfile: PResource;
begin
  DeinitCarpet;
  curfile := PopCurResFile;
  PopUseResFile(levelresources[currentlevel.leveltype]);
  PopSetPalette(3500, 64);
  PopSetPalette(750, 32);
  PopUseResFile(curfile);
end;

procedure InitCarpet;
var
  ctr: Integer;
begin
  new(carpet);
  carpet^.param1 := 0;
  carpet^.param2 := 0;
  carpet^.param3 := 0;
  for ctr := 0 to 9 do
    CheckCarpetImg(ctr);
end;

procedure DeinitCarpet;
var
ctr: integer;
begin
  dispose(carpet);
  carpet := nil;
  for ctr := 0 to 9 do
  begin
  freeMem(carpetshapes[ctr]);
  carpetshapes[ctr] := nil;
  end;
    
end;

function CheckCarpetImg(index: word): pointer;
var
oldres: PResFile;
begin
Result := nil;

if carpetshapes[index]=nil then
begin
oldres := PopCurResFile;
PopUseResFile(levelresources[currentlevel.leveltype]);

carpetshapes[index] := GetShape(25303+index);
PopUseResFile(oldres);
end;
Result := carpetshapes[index];
end;

function GetCarpetShape(index: word): pointer;
begin
  Result := nil;
  if carpet = nil then exit;
  Result := CheckCarpetImg(index);
  
end;

function OffsetBrgFloorY(offset: smallint; col: smallint): smallint;
begin
  Result := custompieces.Content[offset].bottom;
  inc(custompieces.Content[offset].bottom, brgoffsets[col]);
  offsetrect(custompieces.Content[offset].rect, 0, brgoffsets[col]);
end;

procedure RestoreBrgFloorY(idx: smallint; prev: smallint);
var
diff: smallint;
roomtouse: word;
begin
  diff := custompieces.Content[idx].bottom - prev;
  custompieces.Content[idx].bottom := prev;
  offsetrect(custompieces.Content[idx].rect, 0, -diff);
end;

procedure DrawFrontWall;
var
backmasked: word;
rowtouse: smallint;
roomtouse: word;
cell: word;
topback: word;
upped: word;
begin
if (drawingrow <> NextBFItem^.PartRow) or (drawingcol <> NextBFItem^.PartCol) then exit;
backmasked := (NextBFItem^.PartBack.back1 and $F00) shr 8;
if backmasked = 0 then exit;
dec(backmasked);
if backmasked = 9 then
begin

  frontinfos[backmasked][2] := -102;

  if PieceAboveId(NextBFItem^.partcol, NextBFItem^.partrow) = CELL_LOOSE then
  begin
    if NextBFItem^.PartRow = 0 then
    begin
      rowtouse := 2;
      roomtouse := toproom;
    end
    else
    begin
      rowtouse := NextBFItem^.Partrow - 1;
      roomtouse := currentroom;
    end;
    if roomtouse <> 0 then
    begin
      cell := FirstInRow(rowtouse) + NextBFItem^.PartCol;
      topback := currentlevel.backtables[roomtouse-1][cell].back1;
    end
    else
      topback := 0;

    inc(frontinfos[backmasked][2],loosetops[topback and $F]);
  end;

end;

upped := backmasked + 95;

if (NextBFItem^.layer = 0) and (upped = 108) then
begin
  NextBFItem^.ItemID := frontinfos[backmasked][0];
  NextBFItem^.Left := frontinfos[backmasked][1]+1;
  NextBFItem^.Top := frontinfos[backmasked][2];
  AddNextBFItemToTable;
end
else if NextBFItem^.layer = 1 then
begin
  if upped = 108 then
    inc(backmasked);
  NextBFItem^.ItemID := frontinfos[backmasked][0];
  NextBFItem^.Left := frontinfos[backmasked][1];
  NextBFItem^.Top := frontinfos[backmasked][2];
  AddNextBFItemToTable;


end;


end;

procedure DrawTheDebris;
begin
case NextBFItem^.Layer of
1: begin
  NextBFItem^.ItemID := 30;
  NextBFItem^.Left := currentpieces[14].items[2].Left;
  NextBFItem^.Top := currentpieces[14].items[2].Top;
  AddNextBFItemToTable;
end;
2,5: begin
  NextBFItem^.ItemID := 29;
  NextBFItem^.Left := currentpieces[14].items[1].Left;
  NextBFItem^.Top := currentpieces[14].items[1].Top;
  AddNextBFItemToTable;
end;
end;
end;

end.
