unit uCellUtil;

interface

function IsFloor(celltype: Word): boolean;
function IsImpassable(celltype: Word): boolean;
function IsLandOnable(celltype: Word): boolean;
function IsMobFloor(celltype: Word): boolean;
function IsSpace(celltype: Word): boolean;
function IsWall(celltype: Word): boolean;
function GetRow(Y: smallint): smallint;
function GetCol(X: smallint): smallint;
function IndexCell1(col, row: smallint): smallint;
function IndexCell(col, row: smallint): smallint;
procedure ConvertX(oldx: smallint; var newx: smallint);
function ReadCell(room: word; col: smallint; row: smallint): word;
function Handler: word;
function GetUndefineCellId(col: smallint; row: smallint): word;
function GetCellsInFront(count: word): word;
function GetFirstCellInBack: word;
function ReadCellOnCharBlock: word;


const
CELL_LEFT: array [-1..11] of smallint = (-$33, 0, $33, $66, $99, $CC, $FF, $132, $165, $198, $1CB, $1FE, $231);
CELL_BOTTOM: array [-1..3] of smallint = (5, $7D, $F5, $16D, $1E5);

ftbl: array[-1..0] of smallint = (-1, 1);
btbl: array[-1..0] of smallint = (1, -1);

var
HandlingRoom: word = 0;
HandlingCol: smallint = 0;
HandlingRow: smallint = 0;
HandlingFore: word = 0;
HandlingBack: word = 0;
HandlingCell: smallint;

ColOffset: smallint;

implementation

uses uPop2Types, uRoomUtil, uGlobals;

function IsFloor(celltype: Word): boolean;
begin
Result := not (IsWall(celltype) or IsSpace(celltype))
end;

function IsImpassable(celltype: Word): boolean;
begin
Result := celltype in [CELL_LOOSE, CELL_DROP, CELL_FLIP, CELL_SLICERLEFT, CELL_SLICERRIGHT, CELL_LAVALEFT, CELL_LAVARIGHT];
end;

function IsLandOnable(celltype: Word): boolean;
begin
Result := IsFloor(celltype) and (not IsImpassable(celltype)) and (celltype <> CELL_LOWERED) and (celltype <> CELL_PRESSURE)
end;

function IsMobFloor(celltype: Word): boolean;
begin
Result := celltype in [CELL_LOOSE, CELL_DROP, CELL_FLIP];
end;

function IsSpace(celltype: Word): boolean;
begin
Result := celltype in [CELL_EMPTY, CELL_TOPPILLAR, CELL_CAVEBARS, CELL_WATERPILLAR, CELL_WINDOW];
end;

function IsWall(celltype: Word): boolean;
begin
Result := celltype in [CELL_WALL, CELL_SPIKES, CELL_TUNNEL, CELL_SLIDEWALL, CELL_FLAME];
end;

function GetRow(Y: smallint): smallint;
begin
  dec(Y, 5);
  Result := Y div 120;
  if Y <= 0 then
    Dec(Result);  
end;

function GetCol(X: smallint): smallint;
begin
  Result := (X-4) div 51 - 4;
  ColOffset := (X-4) mod 51;
  if X < 0 then
  dec(result);
end;

function IndexCell1(col, row: smallint): smallint;
begin
  Result := IndexCell(col, row);
  if Result < 0 then
    Result := 30;
end;

function IndexCell(col, row: smallint): smallint;
begin
  if (row >= 3) or (row < -1) or (col > 10) or (col < 0) then
    Result := 30
  else if row = -1 then
    Result := -(col+1)
  else
    Result := FirstInRow(row)+col;
end;

procedure ConvertX(oldx: smallint; var newx: smallint);
begin
  newx := oldx;
  if oldx < 0 then
    dec(newx);
end;

function ReadCell(room: word; col: smallint; row: smallint): word;
begin
  if (col<>HandlingCol) or (room<>HandlingRoom) or (row<>HandlingRow) then
  begin
    HandlingRoom := room;
    HandlingCol := col;
    HandlingRow := row;
    HandlingRoom := Handler;
    HandlingCell := FirstInRow(HandlingRow)+HandlingCol;
  end;
  if (HandlingRoom <> 0) and (HandlingRoom <= currentlevel.MaxRooms) then
  begin
    CalcBlue(HandlingRoom);
    handlingfore := currentforetable^[HandlingCell];
    handlingback := currentbacktable^[handlingcell].back1; 
  end
  else
  begin
    HandlingBack := 0;
    HandlingFore := GetUndefineCellId(col, row);
  end;

  Result := HandlingFore;

end;

function Handler: word;
var
room: word;
col: smallint;
begin
room := HandlingRoom;
col := 0;
if HandlingCol < 0 then
begin
  col := 10;
  if HandlingRoom <> 0 then
  begin
    room := currentlevel.relations[room-1].Left;
    if room <> 0 then
    begin
      inc(HandlingCol,10);
      col := 0;
      HandlingRoom := room;
      Handler;
    end;
  end;
end
else if HandlingCol >= 10 then
begin
  col := -10;
  if HandlingRoom <> 0 then
  begin
    room := currentlevel.relations[room-1].Right;
    if room <> 0 then
    begin
      dec(HandlingCol,10);
      col := 0;
      HandlingRoom := room;
      Handler;
    end;
  end;
end;


if HandlingRoom <> 0 then
begin
  if HandlingRow < 0 then
  begin
    inc(HandlingRow, 3);
    room := currentlevel.relations[room-1].Up;
    HandlingRoom := room;
    Handler;
  end
  else if HandlingRow >= 3 then
  begin
    dec(HandlingRow, 3);
    room := currentlevel.relations[room-1].Down;
    HandlingRoom := room;
    Handler;
  end;
end;

if (room = 0) and (HandlingRoom <> 0) then
  HandlingRoom := 0;

if not (HandlingCol in [0..10]) then
  inc(handlingcol, col);

Result := HandlingRoom;

end;

function GetUndefineCellId(col: smallint; row: smallint): word;
begin
  case currentlevel.leveltype of
  2: begin
    if (currentroom = 4) and (currentlevel.levelnumber in [4,13]) then
      Result := CELL_EMPTY
    else
      Result := CELL_WALL;
  end;
  4: begin
    if (toproom = 0) and (row = -1) then
      Result := CELL_EMPTY
    else if ((currentroom=27) and (currentlevel.levelnumber=6)) or ((currentroom=16) and (currentlevel.levelnumber=9)) then
      Result := CELL_EMPTY
    else
      Result := CELL_WALL;
  end;
  5: Result := 0;
  6: begin

  if ((currentroom in [7,8]) and (currentlevel.leveltype = 6) and (row<3)) or
   ((currentroom = 3) and (currentlevel.leveltype=6) and ((col < 0) or (col=9)) and (row=2)) then
   Result := CELL_EMPTY
  else
    Result := CELL_WALL;

  end;
  else
    Result := CELL_WALL;
  end;
end;

function GetCellsInFront(count: word): word;
begin
  Result := ReadCell(ActiveChar.Room, ActiveChar.BlockX + ftbl[ActiveChar.Face] * count, ActiveChar.BlockY);
end;

function GetFirstCellInBack: word;
begin
  Result := ReadCell(ActiveChar.Room, ActiveChar.BlockX + btbl[ActiveChar.Face], ActiveChar.BlockY);
end;

function ReadCellOnCharBlock: word;
begin
  Result := ReadCell(ActiveChar.Room, ActiveChar.BlockX, ActiveChar.BlockY);
end;



end.
