unit uColorTables;

interface

uses Windows, uMacFile;

function GetCTBLResource(id: word): PResource;
procedure RemapCTBL(ctbl: pointer; baseindex: word);
procedure SetupPalette();
procedure SetForeColor(index: word);
procedure SetBackColor(index: word);
procedure StandardColors(direct: boolean; color1: word; color2: word);
procedure ShowColors(baseindex: smallint; count: word);
procedure SetColors(ctbl: pointer; flag: word; arg1, arg2: word);
procedure SetCTBLColor(ctblid: smallint; remapbase: smallint);
procedure SetSHAPColors(ctblid: smallint);
procedure PoPSetPalette(ctblid: smallint; remapbase: smallint);
procedure SetKidPalette(leveltype: shortint);
procedure SetRoomPalette(leveltype: shortint);


procedure InitPalette;
procedure DeInitPalette;
//procedure UpdatePalette;
function GetPalette: HPalette;
function GetBWPalette: HPalette;
procedure LoadTable(id: word);
procedure ResetStandardCOlors;


implementation

uses uResources, uSwapper, uColor, uGlobals;

var
pal: PLOGPALETTE = nil;
macpal: array [0..255] of TPaletteEntry;
bwpal: PLOGPALETTE = nil;

hpal: HPalette = 0;
hbwpal: HPalette = 0;
paldirty: boolean = true;


function GetCTBLResource(id: word): PResource;
begin
  Result := PoPGetResource('CTBL', id);
end;

procedure InitPalette;
var
palettesize: integer;
begin
PaletteSize := sizeof(TLogPalette) + (sizeof(TPaletteEntry) * 255);
GetMem(pal,palettesize);
pal^.palVersion := $300;
pal^.palNumEntries := 256;

GetMem(bwpal,palettesize);
bwpal^.palVersion := $300;
bwpal^.palNumEntries := 256;

end;

procedure DeInitPalette;
begin
FreeMem(pal);
DeleteObject(hpal);
hpal := 0;

FreeMem(bwpal);
DeleteObject(hbwpal);
hbwpal := 0;
end;

function GetPalette: HPalette;
begin
  if true or paldirty then
  begin
    if hpal <> 0 then
    DeleteObject(hpal);
    hpal := CreatePalette(pal^);
  end;
  paldirty := false;
  Result := hpal;
end;

function GetBWPalette: HPalette;
begin
  if true or paldirty then
  begin
    if hbwpal <> 0 then
      DeleteObject(hbwpal);
    hbwpal := CreatePalette(bwpal^);
  end;
  Result := hbwpal;
end;

procedure LoadTable(id: word);
type
pfastrgb = ^fastrgb;
fastrgb = packed record
r: byte;
g: byte;
b: byte;
i: byte;
end;
var
res: PResource;
resiter: pbyte;
nbitems: word;
r: fastrgb;
  ctr: Integer;
begin
  res := GetCTBLResource(id);
  resiter := res^.data;
  nbitems := swap2(pword(resiter)^);
  inc(resiter, 2);
  for ctr := 1 to nbitems do
  begin
    r := pfastrgb(resiter)^;
    pal^.palPalEntry[r.i].peRed := r.r;
    pal^.palPalEntry[r.i].peGreen := r.g;
    pal^.palPalEntry[r.i].peBlue := r.b;
    macpal[r.i].peRed := r.r;
    macpal[r.i].peGreen := r.g;
    macpal[r.i].peBlue := r.b;
    inc(resiter, 4);
  end;
  paldirty := true;
end;

procedure RemapCTBL(ctbl: pointer; baseindex: word);
var
resiter: pbyte;
count: word;
  ctr: Integer;
begin
  if ctbl = nil then exit;
  resiter := ctbl;
  count := swap2(pword(resiter)^);
  if count = 0 then exit;
  inc(resiter, 5);
  if resiter^ <> baseindex then
  begin
    for ctr := 0 to count - 1 do
    begin
      resiter^ := baseindex + ctr;
      inc(resiter, 4);
    end;
  end;
end;

procedure SetupPalette();
var
palettesize: integer;
  ctr: Integer;
begin
InitPalette;
ResetStandardColors;

for ctr := 0 to 255 do
begin
bwpal^.palPalEntry[ctr].peRed := ctr;
bwpal^.palPalEntry[ctr].peGreen := ctr;
bwpal^.palPalEntry[ctr].peBlue := ctr;
end;
end;

procedure ResetStandardColors;
begin
  macpal[0].peRed := $FF;
  macpal[0].peGreen := $FF;
  macpal[0].peBlue := $FF;
  macpal[1].peRed := $0;
  macpal[1].peGreen := $0;
  macpal[1].peBlue := $0;
  macpal[2].peRed := $DD;
  macpal[2].peGreen := $0;
  macpal[2].peBlue := $0;
  macpal[3].peRed := $FF;
  macpal[3].peGreen := $FF;
  macpal[3].peBlue := $0;
  macpal[4].peRed := $0;
  macpal[4].peGreen := $0;
  macpal[4].peBlue := $DD;
  macpal[5].peRed := $AA;
  macpal[5].peGreen := $56;
  macpal[5].peBlue := $00;
  macpal[6].peRed := $FD;
  macpal[6].peGreen := $FF;
  macpal[6].peBlue := $A8;
  macpal[7].peRed := $D9;
  macpal[7].peGreen := $7;
  macpal[7].peBlue := $0;

  ShowColors(0, 8);
end;

procedure SetForeColor(index: word);
begin
forecolor := macpal[index];
end;

procedure SetBackColor(index: word);
begin
backcolor := macpal[index];
end;

procedure StandardColors(direct: boolean; color1: word; color2: word);
begin
  if direct then
  begin
    SetForeColor(color1);
    SetBackColor(color2);
  end
  else
  begin
    SetForeColor(color2);
    SetBackColor(color1);
  end;

end;

procedure ShowColors(baseindex: smallint; count: word);
var
  ctr: Integer;
begin
  if baseindex < 0 then exit;
  if baseindex + count <= 256 then
  begin
    for ctr := baseindex to baseindex + count - 1 do
      pal^.palPalEntry[ctr] := macpal[ctr];
    paldirty := true;
  end;
end;

procedure SetColors(ctbl: pointer; flag: word; arg1, arg2: word);
type
pfastrgb = ^fastrgb;
fastrgb = packed record
r: byte;
g: byte;
b: byte;
i: byte;
end;
var
resiter: pbyte;
nbitems: word;
r: fastrgb;
  ctr: Integer;
begin
  resiter := ctbl;
  nbitems := swap2(pword(resiter)^);
  inc(resiter, 2);
  for ctr := 1 to nbitems do
  begin
    r := pfastrgb(resiter)^;
    pal^.palPalEntry[r.i].peRed := r.r;
    pal^.palPalEntry[r.i].peGreen := r.g;
    pal^.palPalEntry[r.i].peBlue := r.b;
    macpal[r.i].peRed := r.r;
    macpal[r.i].peGreen := r.g;
    macpal[r.i].peBlue := r.b;
    inc(resiter, 4);
  end;
  paldirty := true;

end;

procedure SetCTBLColor(ctblid: smallint; remapbase: smallint);
var
res: PResource;
data: pointer;
begin
res := GetCTBLResource(ctblid);
if res <> nil then
begin
  GetMem(data, res^.size);
  move(res^.data, data^, res^.size);
  if remapbase >= 0 then
    RemapCTBL(data, remapbase);
  SetColors(data, 7, 1, 1);
  ShowColors(pbyte(integer(data) + 5)^, swap2(pword(data)^));
  FreeMem(data);
end;
end;

procedure SetSHAPColors(ctblid: smallint);
var
res: PResource;
begin
res := GetCTBLResource(ctblid);
if res <> nil then
begin
  SetColors(res^.data, 7, 1, 1);
  ShowColors(pbyte(integer(res^.data) + 5)^, swap2(pword(res^.data)^));
end;
end;

procedure PoPSetPalette(ctblid: smallint; remapbase: smallint);
var
res: PResource;
data: pointer;
src: pointer;
dest: pointer;
base2: byte;
curfile: PResFile;
begin
res := GetCTBLResource(ctblid);
if res = nil then
begin
curfile := PopCurResFile;
PopUseResFile(resOpp);
res := GetCTBLResource(ctblid);
PopUseResFile(curfile);
end;
if res = nil then
begin
curfile := PopCurResFile;
PopUseResFile(resKid);
res := GetCTBLResource(ctblid);
PopUseResFile(curfile);
end;
if res = nil then
begin
curfile := PopCurResFile;
PopUseResFile(resPrince);
res := GetCTBLResource(ctblid);
PopUseResFile(curfile);
end;

getmem(data, res^.size);
move(res^.Data^, data^, res^.size);
if remapbase >= 0 then
  RemapCTBL(data, remapbase);
if roomcolors <> nil then
begin
  src := pointer(integer(data) + 2);
  base2 := pbyte(integer(data) + 5)^;
  dest := pointer(integer(roomcolors) + 2 + base2 * 4);
  move(src^, dest^, swap2(pword(data)^));
end
else
begin
  SetColors(data, 7, 1, 1);
  ShowColors(remapbase, swap2(pword(data)^));
end;
FreeMem(data);
end;

procedure SetKidPalette(leveltype: shortint);
begin
  if (leveltype > 0) and (leveltype < 7) then
  PoPSetPalette(25000+leveltype, 16);
  
end;

procedure SetRoomPalette(leveltype: shortint);
begin
  case leveltype of
  2: begin
    if currentlevel.levelnumber = 10 then
    begin
      if currentroom in [22,25,26] then
        PopSetPalette(25361, 1)
      else
        PopSetPalette(20, 1);
    end;
  end;
  4: begin
    if currentlevel.levelnumber = 9 then
    begin
      if currentroom in [2, 11, 12, 13, 14, 15, 16] then
        PopSetPalette(25360, 1)
      else
        PopSetPalette(20, 1);
    end;
  end;
  5: begin
    if currentroom in [15,16,19] then
      PopSetPalette(3501, 64)
    else
      PopSetPalette(3500, 64);
  end;
  6: begin
    if (currentroom = 1) and (currentlevel.levelnumber = 14) then
      PopSetPalette(25363, 1)
    else if ((toproom = 1) and (currentlevel.levelnumber = 14)) or ((currentroom = 11) and (currentlevel.levelnumber = 14)) then
      PopSetPalette(20, 1);
  end;
  end;
end;

end.
