unit uCustom;

interface

uses uPop2Types, uMacTypes;

procedure RunTemplateIn;
procedure RunTemplateOut;
procedure UnloadCustomRoomShapes;
procedure ClearCustomShapesTable;
procedure LoadCustomRoomShapes(data: PCustomPieces);
procedure LoadCustomRoomData(room: word);
procedure CustomPieceOffsetToColRow(piece: PCustomPiece; col: smallint; row: smallint);
procedure GetCustomPieceSize(index: word; var size: TMacPoint);

implementation

uses Math, uGlobals, uMacFile, uResources, uSwapper, uRect, uShapes, uCaves, uRuins, uTemple, uFinal;

var

templates: array[0..$23] of array[0..1] of procedure = (
(uCaves.CarpetRoomTemplateIn,uCaves.CarpetRoomTemplateOut),
(nil,nil),
(nil,nil),
(nil,nil),
(nil,nil),
(nil,nil),
(nil,nil),
(nil,nil),
(nil,nil),
(uFinal.PrincessRoomTemplateIn,nil),
(nil,nil),
(nil,nil),
(nil,nil),
(nil,nil),
(nil,nil),
(nil,nil),
(nil,nil),
(nil,nil),
(nil,nil),
(nil,nil),
(nil,nil),
(nil,nil),
(uRuins.EntranceRoomTemplateIn,uRuins.EntranceRoomTemplateOut),
(uFinal.TowerTopTemplateIn,nil),
(uFinal.TowerBottomTemplateIn,nil),
(uFinal.SpiderAndChessRoomTemplateIn,nil),
(uFinal.SpiderAndChessRoomTemplateIn,nil),
(uFinal.SpiderAndChessRoomTemplateIn,nil),
(uFinal.CrystalAndEscherRoomTemplateIn,nil),
(uFinal.CrystalAndEscherRoomTemplateIn,nil),
(uFinal.CrystalAndEscherRoomTemplateIn,nil),
(nil,nil),
(uTemple.AltarRoomTemplateIn,uTemple.AltarRoomTemplateOut),
(nil,nil),
(uRuins.ThroneRoomTemplateIn,uRuins.EntranceRoomTemplateOut),
(nil,nil)
);

procedure RunTemplateIn;
begin
  if assigned(templates[custompieces^.ID][0]) then
    templates[custompieces^.ID][0];
end;

procedure RunTemplateOut;
begin
  if assigned(templates[custompieces^.ID][1]) then
    templates[custompieces^.ID][1];
end;

procedure UnloadCustomRoomShapes;
var
done: array of boolean;
  ctr: Integer;
begin
if custompieces = nil then exit;
SetLength(done, custompieces^.MaxID + 1);
FillChar(done[0], custompieces^.MaxID + 1, 0);
for ctr := 0 to custompieces^.nbitems - 1 do
begin
  if not done[custompieces^.Content[ctr].index] then
  begin
    done[custompieces^.Content[ctr].index] := true;
    FreeMem(custompieces^.Content[ctr].shape);
    custompieces^.Content[ctr].shape := nil;
  end;
end;


SetLength(done, 0);
end;

procedure ClearCustomShapesTable;
begin
  if custompieces = nil then exit;
  UnloadCustomRoomShapes;
  RunTemplateOut;
  FreeMem(custompieces);
  custompieces := nil;


end;

procedure LoadCustomRoomShapes(data: PCustomPieces);
var
backing: array of pointer;
saved: array of pointer;
limit: word;
  ctr: Integer;
itemtop: smallint;
temprect: TMacRect;
temprect2: TMacRect;
begin

if custompieces <> nil then
begin
  if data^.FirstID = custompieces^.FirstID then
  begin
    limit := max(custompieces^.MaxID, data^.MaxID);
    SetLength(backing, limit+1);
    FillChar(backing[0], 4*(limit+1), 0);
    for ctr := 0 to custompieces^.NbItems - 1 do
    begin
      if custompieces^.Content[ctr].shape <> nil then
      begin
        backing[custompieces^.Content[ctr].index] := custompieces^.Content[ctr].shape;
        custompieces^.Content[ctr].shape := nil;
      end;
    end;

  end;
  if data^.ID <> custompieces^.ID then
    ClearCustomShapesTable;
end;

SetLength(saved, data^.MaxID + 1);
FillChar(saved[0], 4*(data^.MaxID+1), 0);
for ctr := 0 to data^.NBItems - 1 do
begin
  if data^.Content[ctr].index <> -1 then
  begin
    with data^.Content[ctr] do
    begin
      if saved[index] = nil then
      begin
        if (length(backing) > 0) and (backing[index] <> nil) then
        begin
          saved[index] := backing[index];
          backing[index] := nil;
        end
        else
        begin
          saved[index] := GetShape(data^.firstid + index);
        end;
      end;
      shape := saved[index];
      itemtop := bottom - swap2(pword(integer(saved[index]) + 6)^);
      MakeShapeRect(shape, itemtop, left, temprect);
      if data^.Precalculated = 0 then
      begin
        if EmptyRect(rect) then
          move(temprect,rect,sizeof(TMacRect))
        else
        begin
          inc(rect.top, itemtop);
          inc(rect.bottom, itemtop);
          inc(rect.left, left);
          inc(rect.right, left);
          SectRect(rect, temprect, temprect2);
          move(temprect2,rect,sizeof(TMacRect));
        end;
      end;
      shapeid := data^.FirstID + ctr;
    end;
  end
  else //index -1
  begin
    data^.Content[ctr].shape := nil;
    data^.Content[ctr].shapeid := -1;
    data^.Content[ctr].rect.top := 0;
    data^.Content[ctr].rect.left := 0;
    data^.Content[ctr].rect.bottom := 0;
    data^.Content[ctr].rect.right := 0;
  end;
end;

for ctr := 0 to length(backing) - 1 do
begin
  FreeMem(backing[ctr]);
end;

SetLength(backing, 0);
SetLength(saved, 0);
end;

procedure LoadCustomRoomData(room: word);
var
res: PResource;
pcust: PCustomPieces;
resdata: psmallint;
  ctr: Integer;
begin
if customized then
begin
  res := PoPGetResource('CUST',(room-1)*25+4000);
  if res = nil then exit;
    resdata := res^.data;
    new(pcust);
    pcust^.ID := swap2(pword(resdata)^);
    inc(resdata);
    pcust^.FirstID := swap2(pword(resdata)^);
    inc(resdata);
    pcust^.MaxID := swap2(pword(resdata)^);
    inc(resdata);
    pcust^.Precalculated := swap2(pword(resdata)^);
    inc(resdata, 4);
    pcust^.Var_E := swap2(resdata^);
    inc(resdata, 7);
    pcust^.NBItems := swap2(pword(resdata)^);
    inc(resdata);
    setlength(pcust^.Content, pcust^.NBItems);
    for ctr := 0 to pcust^.nbitems - 1 do
    begin
      pcust^.Content[ctr].index := swap2(pword(resdata)^);
      inc(resdata);
      pcust^.Content[ctr].bottom := swap2(resdata^);
      inc(resdata);
      pcust^.Content[ctr].left := swap2(resdata^);
      inc(resdata);
      pcust^.Content[ctr].layer := swap2(pword(resdata)^);
      inc(resdata);
      pcust^.Content[ctr].transparent := swap2(pword(resdata)^);
      inc(resdata);
      pcust^.Content[ctr].shapeid := swap2(pword(resdata)^);
      inc(resdata);
      pcust^.Content[ctr].shape:= nil;
      inc(resdata, 2);
      pcust^.Content[ctr].rect.top := swap2(resdata^);
      inc(resdata);
      pcust^.Content[ctr].rect.left := swap2(resdata^);
      inc(resdata);
      pcust^.Content[ctr].rect.bottom := swap2(resdata^);
      inc(resdata);
      pcust^.Content[ctr].rect.right := swap2(resdata^);
      inc(resdata);
      inc(resdata);
      pcust^.Content[ctr].var_1A := swap2(resdata^);
      inc(resdata, 3);
    end;

    if (custompieces = nil) or (custompieces^.ID <> pcust^.ID) then
    begin
      LoadCustomRoomShapes(pcust);
      ClearCustomShapesTable;
      custompieces := pcust;
      //SetCustomRoomTemplate
    end
    else
    begin
      LoadCustomRoomShapes(pcust);
      ClearCustomShapesTable;
      custompieces := pcust;
    end;
    RunTemplateIn;
    pcust^.Precalculated := 1;
end
else
begin
  if custompieces <> nil then
    ClearCustomShapesTable;
end;
end;

procedure CustomPieceOffsetToColRow(piece: PCustomPiece; col: smallint; row: smallint);
begin
inc(piece^.bottom, (row+1)*120);
inc(piece^.left, (col*51));
OffsetRect(piece^.rect, piece^.left, piece^.bottom);

end;

procedure GetCustomPieceSize(index: word; var size: TMacPoint);
begin
  size.x := 0;
  size.y := 0;
  if custompieces = nil then exit;
  if custompieces^.Content[index].shape = nil then exit;
  size.x := swap2(pword(integer(custompieces^.Content[index].shape)+4)^);
  size.y := swap2(pword(integer(custompieces^.Content[index].shape)+6)^);
end;

end.

