unit uImgTables;

interface

uses uRect, uPop2Types, uMacTypes;

type
PBFImgEntry = ^TBFImgEntry;
TBFImgEntry = record
  ImgSet: word;
  Col: smallint;
  Row: smallint;
  Transparent: word;
  Flip: word;
  Layer: word;
  UnkC: word;
  PartCol: smallint;
  PartRow: smallint;
  PartFore: word;
  PartBack: TBackPair;
  ItemID: word;
  Left: smallint;
  Top: SmallInt;
  ClippedRect: TMacRect;
  IsCustom: boolean;
end;

PMidImgEntry = ^TMidImgEntry;
TMidImgEntry = record
  Left: smallint;
  Top: smallint;
  ImgSet: smallint;
  ImgID: smallint;
  Rect: TMacRect;
  Transparency: word;
  ColorOffset: Word;
  Flip: Word;
end;

var
ForeImgTable: array [0..119] of TBFImgEntry;
BackImgTable: array [0..129] of TBFImgEntry;
MidImgTable1: array [0..29] of TMidImgEntry;
MidImgTable2: array [0..29] of TMidImgEntry;
NextBFItem: PBFImgEntry;
ForeImgCount: word;
BackImgCount: word;
MidImgCount1: word;
MidImgCount2: word;

procedure ResetNextBFItem(layer: word);
procedure TransferBFItemMidFields(src: PBFImgEntry; dest: PBFImgEntry);
procedure ResetNextBFItemWithPrevInfo(layer: word);
function AddNextBFItemToTable: boolean;
function GetBackImgStartPass: word;
procedure CheckNextBFItem(layer: word);
procedure DrawBackImg(layer: word);
procedure RedrawCellSure;
procedure RedrawCellFast(cell: word);
procedure DrawForeImg;
procedure Sure;
procedure Fast;
procedure ClearBackTable;
procedure ClearMidTable;
procedure DoSure;
procedure DoFast;
procedure DrawCustomRoomImgs(layer: word);
procedure DrawCustomRoomImg(index: smallint; layer: word);
procedure DrawHorse(point: PMacPoint; layer: word);
procedure DrawCustomRoomImgAtInfo(index: smallint);
function GetBackTableImgRow(imgid: word; col, row: smallint): PBFImgEntry;
function AddMid(ImgSet: smallint; ImgID: smallint; X, Y: smallint; Transparency: word): boolean;


implementation

uses uGlobals, uShapeSets, uShapeList, uSwapper, uCellUtil, uLevelDraw, uRoomUtil, uCustom, uObj;

procedure ResetNextBFItem(layer: word);
begin
  if layer in [1..2] then
  begin
    if ForeImgCount < 120 then
      NextBFItem := @ForeImgTable[ForeImgCount]
    else
      NextBFItem := @ForeImgTable[119];
  end
  else
  begin
    if BackImgCount < 130 then
      NextBFItem := @BackImgTable[BackImgCount]
    else
      NextBFItem := @BackImgTable[129];
  end;
  NextBFItem^.ImgSet := 4;
  NextBFItem^.Col := DrawingCol;
  NextBFItem^.Row := DrawingRow;
  NextBFItem^.Transparent := $10;
  NextBFItem^.Flip := 0;
  NextBFItem^.Layer := layer;
  NextBFItem^.UnkC := 0;
end;

procedure TransferBFItemMidFields(src: PBFImgEntry; dest: PBFImgEntry);
begin
  dest^.PartCol := src^.PartCol;
  dest^.PartRow := src^.PartRow;
  dest^.PartFore := src^.PartFore;
  dest^.PartBack := src^.PartBack;
end;

procedure ResetNextBFItemWithPrevInfo(layer: word);
var
prev : PBFImgEntry;
begin
ResetNextBFItem(layer);
if layer in [1..2] then
  prev := @ForeImgTable[ForeImgCount - 1]
else
  prev := @BackImgTable[BackImgCount - 1];

TransferBFItemMidFields(prev, NextBFItem);
end;

function AddNextBFItemToTable: boolean;
var
shape: pointer;
rect: TMacRect;
clipped: TMacRect;
begin
  if NextBFItem <> nil then
  begin
    if NextBFItem^.ImgSet = 4 then
      shape := GetBGSetShapeItem(NextBFItem^.ItemID)
    else
    begin
      shape := PrinceGetnShapeItem(shapesets[NextBFItem^.ImgSet],NextBFItem^.ItemID);
    end;
  end;
  if (shape <> nil) and (swap2(pword(integer(shape)+6)^) > 0) then
  begin
    inc(NextBFItem^.Top, CELL_BOTTOM[NextBFItem^.PartRow] - swap2(pword(integer(shape)+6)^));
    inc(NextBFItem^.Left, CELL_LEFT[NextBFItem^.PartCol]);
    MakeShapeRect(shape, NextBFItem^.Top, NextBFItem^.Left, rect);
    Result := SectRect(DrawingRect, rect, clipped);
    if Result then
    begin
      NextBFItem^.ClippedRect := clipped;
      if NextBFItem^.Layer in [1,2] then
        inc(ForeImgCount)
      else
        inc(BackImgCount);
      NextBFItem := nil;
    end;
  end
  else
    Result := false;
end;

function GetBackImgStartPass: word;
var
step: boolean;
begin
  if drawingfull then
  begin
    if (os_fore = CELL_TUNNEL) or (not IsWall(os_fore)) or (OS_LeftFore = CELL_TUNNEL) or (not IsWall(OS_LeftFore))  then
    begin
      if (drawingcol = 0) or (drawingrow = 2) then
        step := false
      else
        step := true;
    end
    else
      step := true;
  end
  else
    step := false;
  if step then
    Result := 3
  else
  begin
    if (not drawingfull) or (drawingcol <> 0) or (drawingrow = 2) then
    begin
      if (custompieces <> nil) and (drawingrow=2) and (custompieces^.ID <> $1F) then
        Result := 2
      else
        Result := 0;
    end
    else
      Result := 2;
  end;
end;

procedure CheckNextBFItem(layer: word);
begin
  if NextBFItem = nil then
    ResetNextBFItem(layer);
end;

procedure DrawBackImg(layer: word);
var
pass: word;
willdraw: boolean;
myback: word;
begin
pass := GetBackImgStartPass;
ResetNextBFItem(layer);
if drawingfull then
drawingrect := backbuffer.Rect;
repeat
CheckNextBFItem(layer);

if pass = 0 then
begin
  NextBFItem^.PartCol := drawingcol - 1;
  NextBFItem^.PartRow := drawingrow + 1;
  NextBFItem^.PartFore := below[drawingcol].Fore;
  NextBFItem^.PartBack := below[drawingcol].Back;
  if drawingfull and (drawingcol <> 0) and (drawingcol<>9) and
  (currentlevel.leveltype <> 3) and (NextBFItem^.PartFore <> CELL_DOOR) and
  (currentlevel.levelnumber <> 13) and
  ((currentlevel.leveltype<>2) or (not (NextBFItem^.PartFore in [CELL_TORCH,CELL_TOPPILLAR])))  then
  inc(pass,2);
end
else if pass = 1 then
begin
  NextBFItem^.PartCol := drawingcol;
  NextBFItem^.PartRow := drawingrow +1;
  NextBFItem^.PartFore := below[drawingcol+1].Fore;
  NextBFItem^.PartBack := below[drawingcol+1].Back;
  if IsWall(NextBFItem^.PartFore) then
  begin
    inc(pass,1);
    NextBFItem^.PartCol := drawingcol -1;
    NextBFItem^.PartRow := drawingrow;
    NextBFItem^.PartFore :=OS_LeftFore;
    NextBFItem^.PartBack := OS_LeftBack;
  end;
end
else if pass = 2 then
begin
  NextBFItem^.PartCol := drawingcol -1;
  NextBFItem^.PartRow := drawingrow;
  NextBFItem^.PartFore :=OS_LeftFore;
  NextBFItem^.PartBack := OS_LeftBack;
end
else if pass = 3 then
begin
  DrawCustomRoomImgs(layer);
  CheckNextBFItem(layer);
  NextBFItem^.PartCol := drawingcol;
  NextBFItem^.PartRow := drawingrow;
  NextBFItem^.PartFore :=OS_Fore;
  NextBFItem^.PartBack := OS_Back;

end;
myback := NextBFItem^.PartBack.Back1;
myback := (myback and $C000) shr 14;
if (custompieces <> nil) and (myback <> 0) and (myback <> layer) and ((myback <> 2) or (layer <> 5)) then
  willdraw := false
else
  willdraw := true;

if willdraw then
  DrawItem(NextBFItem^.PartFore);

inc(pass);
until pass >= 4;
end;

procedure RedrawCellSure;
begin
  DrawBackImg(0);
  DrawBackImg(5);
  if custompieces <> nil then
    DrawBackImg(11);
  DrawForeImg;
end;

procedure RedrawCellFast(cell: word);
begin

end;

procedure DrawForeImg;
var
pass: word;
myback: word;
begin
  ResetNextBFItem(1);
  //CheckGateInFore //pas besoin: on ne dessine pas les persos.
  if ((currentlevel.LevelType = 5) and (OS_LeftFore = CELL_FLOOR) and (OS_Fore = CELL_EMPTY)) or
  ((currentlevel.LevelType = 4) and ((OS_LeftBack.Back1 and $F00) <> 0) and ((OS_LeftBack.Back1 and $F00) < $D00)) or
  ((currentlevel.LevelType = 2) and (OS_LeftFore = CELL_BOTTOMPILLAR))   then
    pass := 0
  else
    pass := 1;

repeat

CheckNextBFItem(1);

case pass of
0: begin
  NextBFItem^.PartCol := drawingcol - 1;
  NextBFItem^.PartRow := drawingrow;
  NextBFItem^.PartFore := OS_LeftFore;
  NextBFItem^.PartBack := OS_LeftBack;
end;
1: begin
  NextBFItem^.PartCol := drawingcol;
  NextBFItem^.PartRow := drawingrow;
  NextBFItem^.PartFore := OS_Fore;
  NextBFItem^.PartBack := OS_Back;
end;
end;

myback := (NextBFItem^.PartBack.Back1 and $C000) shr $E;
if (custompieces = nil) or (myback in [0..2]) then
  DrawItem(nextbfitem^.PartFore);


inc(pass);
until pass >= 2;

DrawCustomRoomImgs(1);

end;

procedure Sure;
var
current: word;
ctr1, ctr2: smallint;
begin
  ComputeImmediate;
  for ctr1 := 2 downto 0 do
  begin
    drawingrow := ctr1;
    GetAboveAry;
    GetBelowAry;
    for ctr2 := 0 to 9 do
    begin
      drawingcol := ctr2;
      SelectObjects;
      RedrawCellSure;
    end;
  end;
  if not customized then
  begin
    current := currentroom;
    currentroom := toproom;
    MapSurroundingRooms;
    bottomroom := current;
    ComputeImmediate;
    DrawingRow := 2;
    GetAboveAry;
    GetBelowAry;
    drawingrow := -1;
    for ctr2 := 0 to 9 do
    begin
      //drawingfull := false;
      drawingcol := ctr2;
      SelectObjects;
      RedrawCellSure;
      //drawingfull := true;
    end;
    currentroom := current;
    MapSurroundingRooms;
  end
  else
  begin
    ClearAboveCustomized;
  end;
end;

procedure Fast;
var
ctr: integer;
  ctr2: Integer;
  current:word;
begin
  ComputeImmediate;
  //DrawObjs(30);
  drawingrow := 2;
  for ctr := 2 downto 0 do
  begin
    drawingrow := ctr;
    GetAboveAry;
    GetBelowAry;
    for ctr2 := 0 to 9 do
    begin
      drawingrow := ctr2;
      SelectObjects;
      RedrawCellFast(FirstInRow(drawingrow)+drawingcol);
    end;
  end;
  if not customized then
  begin
    current := currentroom;
    currentroom := toproom;
    MapSurroundingRooms;
    ComputeImmediate;
    drawingrow := 2;
    GetAboveAry;
    GetBelowAry;
    for ctr := 0 to 9 do
    begin
      drawingcol := ctr;
      SelectObjects;
      //RedrawTopFast(drawingcol);
    end;
    currentroom := current;
    MapSurroundingRooms;
  end
  else
    ClearAboveCustomized;
  //DrawObjs(-1);
    
end;

procedure ClearBackTable;
begin
  fillchar(BackImgTable[0], BackImgCount * sizeof(TBFImgEntry), 0);
  fillchar(ForeImgTable[0], ForeImgCount * sizeof(TBFImgEntry), 0);
  BackImgCount := 0;
  ForeImgCount := 0;
end;

procedure ClearMidTable;
begin
  fillchar(MidImgTable1[0], MidImgCount1 * sizeof(TMidImgEntry), 0);
  fillchar(MidImgTable2[0], MidImgCount2 * sizeof(TMidImgEntry), 0);
  MidImgCount1 := 0;
  MidImgCount2 := 0;
end;


procedure DoSure;
begin
  ClearBackTable;
  ClearMidTable;
  ClearObjTable;
  Sure;
end;

procedure DoFast;
begin
  //d'autres trucs.
  Fast;
end;

procedure DrawCustomRoomImgs(layer: word);
var
candraw: boolean;
ctr: integer;
truc: PMacPoint;
begin
  candraw := false;
  if (layer in [0,5]) and drawingfull and (drawingcol=0) and (drawingrow=2) then
  begin
    candraw := true;
    if (currentroom = 1) and (currentlevel.LevelNumber = 14) then
    begin
      //jsr $121E(a5)
    end
  end
  else if (layer = 0) and (not drawingfull) and (custompieces <> nil) and (custompieces^.Var_E <> 0) then
    candraw := true
  else if (layer in [1..2]) then
  begin
    if ((drawingcol=9) and (drawingrow = 0)) or not drawingfull then
      candraw := true;
  end;
  if not candraw then exit;
  if custompieces <> nil then
  begin
    for ctr := 0 to custompieces^.NbItems - 1 do
      DrawCustomRoomImg(ctr, layer);
  end;
  if (drawingfull and (layer = 0)) or (layer = 1) then
  begin
    truc := GetHorsePtInPlane(currentroom, layer);
    if truc <> nil then
      DrawHorse(truc, layer);
  end;

end;

procedure DrawCustomRoomImg(index: smallint; layer: word);
var
piece: PCustomPiece;
rect: TMacRect;
img: PBFImgEntry;
begin
piece := @custompieces^.Content[index];

if (piece^.shapeid = -1) or (piece^.layer <> layer) then exit;
if not SectRect(drawingrect, piece^.rect, rect) then exit;
if layer in [0,5,11] then
begin
  if piece^.shape = nil then exit;
  img := @BackImgTable[BackImgCount];
  inc(BackImgCount);
end
else
begin
  img := @ForeImgTable[ForeImgCount];
  inc(ForeImgCount);
end;

img^.ImgSet := 4;
img^.Col := drawingcol;
img^.Row := drawingrow;
img^.Transparent := piece^.Transparent;
img^.Flip := 0;
img^.UnkC := piece^.var_1A;
if NextBFItem <> nil then
begin
img^.PartCol := NextBFItem^.PartCol;
img^.PartRow := NextBFItem^.PartRow;
img^.PartFore := NextBFItem^.PartFore;
img^.PartBack := NextBFItem^.PartBack;
end
else
begin

if layer in [0,5,11] then
begin
img^.PartCol := BackImgTable[BackImgCount-2].PartCol;
img^.PartRow := BackImgTable[BackImgCount-2].PartRow;
img^.PartFore := BackImgTable[BackImgCount-2].PartFore;
img^.PartBack.Back1 := BackImgTable[BackImgCount-2].PartBack.Back1;
img^.PartBack.Back2 := BackImgTable[BackImgCount-2].PartBack.Back2;

end
else
begin
img^.PartCol := ForeImgTable[ForeImgCount-2].PartCol;
img^.PartRow := ForeImgTable[ForeImgCount-2].PartRow;
img^.PartFore := ForeImgTable[ForeImgCount-2].PartFore;
img^.PartBack.Back1 := ForeImgTable[ForeImgCount-2].PartBack.Back1;
img^.PartBack.Back2 := ForeImgTable[ForeImgCount-2].PartBack.Back2;

end;
end;
img^.ItemID := piece^.ShapeId;
img^.Left := piece^.Left;
img^.Top := piece^.Bottom - swap2(pword(integer(piece^.shape) + 6)^);
img^.ClippedRect := rect;

img^.iscustom := true;

NextBFItem := nil;

end;

procedure DrawHorse(point: PMacPoint; layer: word);
var
img: PBFImgEntry;
begin
  if point=nil then exit;
if layer = 0 then
begin
  img := @BackImgTable[BackImgCount];
  inc(BackImgCount);
end
else
begin
  img := @ForeImgTable[ForeImgCount];
  inc(ForeImgCount);
end;

if NextBFItem <> nil then
  NextBFItem := nil
else
begin
  img^.Layer := layer;
  img^.PartCol := drawingcol;
  img^.PartRow := drawingrow;
end;
img^.ImgSet := 4;
img^.Col := drawingcol;
img^.Row := drawingrow;
img^.Transparent := $10;
if currentlevel.LevelNumber = 9 then
img^.Flip := 1
else
img^.Flip := 0;
img^.PartFore := CELL_FLOOR;
img^.Top := point^.y;
img^.Left := point^.x;
img^.ItemID := 25360;
img^.ClippedRect:=drawingrect;


end;

procedure DrawCustomRoomImgAtInfo(index: smallint);
var
oldrect: TMacRect;
piece: PCustomPiece;
oldbottom: smallint;
oldleft: smallint;
oldlayer: word;
layer: word;
begin
piece := @custompieces^.Content[index];
oldrect := piece^.rect;
oldbottom := piece^.bottom;
oldleft := piece^.left;
oldlayer := piece^.layer;
CustomPieceOffsetToColRow(piece, NextBFItem^.Col, NextBFItem^.Row);
layer := NextBFItem^.layer;
piece^.layer := layer;
DrawCustomRoomImg(index, piece^.layer);
if NextBFItem = nil then
  ResetNextBFItemWithPrevInfo(layer);
piece^.bottom := oldbottom;
piece^.left := oldleft;
piece^.rect := oldrect;
piece^.layer := oldlayer;
end;

function GetBackTableImgRow(imgid: word; col, row: smallint): PBFImgEntry;
var
  ctr: Integer;
begin
  Result := nil;
  for ctr := 0 to BackImgCount - 1 do
  begin
    if (BackImgTable[ctr].ItemID = imgid) and (BackImgTable[ctr].col = col) and (BackImgTable[ctr].row = row) then
    begin
      Result := @BackImgTable[ctr];
      exit;
    end;
  end;
end;

function AddMid(ImgSet: smallint; ImgID: smallint; X, Y: smallint; Transparency: word): boolean;
var
mymid: PMidImgEntry;
iid: smallint;
imgsize: TMacPoint;
begin
  Result := false;
  if imgid = 0 then exit;
  if midimgcount1 = 30 then exit;
  if EmptyRect(FCharClipRect) then exit;
  if FCharIdx = 100 then
    mymid := @midimgtable2[midimgcount2]
  else
    mymid := @midimgtable1[midimgcount1];

  mymid^.Left := x;

  iid := imgid-1;

  if (currentlevel.levelnumber = 5) and (currentroom = 10) and (imgset=4) and (iid < custompieces^.NBItems) then
    GetCustomPieceSize(iid, imgsize)
  //else if (length(flameshapes)<> 0) and (iid >= 4077) and (iid <= 4122) then
  //  GetFlameImgSize(iid, imgsize)
  else
  begin
  if FCharSet = 4 then
    GetBGSetImgSize(iid, imgsize)
  else
    PrinceGetnShapeItemSize(shapesets[imgset], iid+1, imgsize);

  end;
  Result := true;

  if FCharFace = 0 then
  begin
    mymid^.Flip := 1;
  end
  else
  begin
    mymid^.Flip := 0;
  end;

  mymid^.Top := y - imgsize.y + 1;

  mymid^.ImgSet := imgset;
  mymid^.ImgID := iid;
  mymid^.Rect := FCharClipRect;
  mymid^.Transparency := transparency;

  if FCharIdx = 100 then
    inc(midimgcount2)
  else
    inc(midimgcount1);


end;

end.
