unit uMacFile;

interface

uses
  Classes, SysUtils;

type

  PResource = ^TResource;
  TResource = record
    id: smallint;
    name: shortstring;
    data: pointer;
    size: cardinal;
  end;

  PResFile = type pointer;

  PResType = array[0..3] of ansichar;

function OpenResFile(filename: string): PResFile;
procedure CloseResFile(handle: PResFile);
function GetResource(resfile: PResFile; _type: PResType; id: integer): PResource;

implementation

uses uSwapper;

type

  PResTypeMap = ^TResTypeMap;
  TResTypeMap = record
    restype: string[4];
    nbentry: cardinal;
    entries: PResource;
  end;

  TResMap = record
    nbtypes: cardinal;
    alltypes: PResTypeMap;
  end;

  _PResFile = ^TResFile;
  TResFile = record
    handle: THandle;
    map: TResMap;
  end;



function ReadUncrypted8(Stream: TStream): string;
var
  size: byte;
  textbuffer: array of char;
begin
  Stream.Read(size, sizeof(byte));
  SetLength(textbuffer, size + 1);
  FillChar(pchar(textbuffer)^, size + 1, 0);
  Stream.Read(pchar(textbuffer)^, size);
  Result := pchar(textbuffer);
end;

function OpenResFile(filename: string): PResFile;
var
  fs: TFileStream;
  toppos: integer;
  resdataoffset: cardinal;
  resmapoffset: cardinal;
  resdatasize: cardinal;
  resmapsize: cardinal;
  rfile: _PResFile;
  typetrack: PResTypeMap;
  restypelistoffset: word;
  resnamelistoffset: word;
  ctr: integer;
  nbtypes: smallint;
  nbres: smallint;
  resoff: word;
  entrytrack: PResource;
  ctr2: integer;
  resid: smallint;
  resnameoffset: word;
  resoffdat: cardinal;
  prevpos2: int64;
  prevpos1: int64;
  resourcesize: cardinal;
  restype: array [0..3] of char;
begin
  Result := nil;
  if not FileExists(filename) then exit;

  fs := TFileStream.Create(filename, fmOpenRead or fmShareDenyWrite);
  {
  fs.Seek(83, soFromBeginning);
  fs.read(datasize, 4);
  fs.read(ressize, 4);
  fs.Seek(120, soFromBeginning);
  fs.read(sechdrsize, 2);
  datasize := (swap4(datasize) + $7F) and $FFFFFF80; //inverse et arrondi
  ressize := swap4(ressize);
  sechdrsize := (swap2(sechdrsize) + $7F) and $FF80; //inverse et arrondi
  if ressize = 0 then
  begin
    fs.free;
    exit;
  end;
  toppos := 128 + sechdrsize + datasize;
  fs.seek(toppos, soFromBeginning);
  }
  fs.read(resdataoffset, 4);
  fs.read(resmapoffset, 4);
  fs.read(resdatasize, 4);
  fs.read(resmapsize, 4);
  resdataoffset := swap4(resdataoffset); //c'est du mac donc on converti
  resmapoffset := swap4(resmapoffset);
  resdatasize := swap4(resdatasize);
  resmapsize := swap4(resmapsize);

  new(rfile);

  toppos := 0;

  fs.seek(toppos + resmapoffset, soFromBeginning); //on se place dans le resmap
  fs.seek(16 + 4 + 2 + 2, soFromCurrent); //copie de l'entete principale

  fs.read(restypelistoffset, 2);
  fs.read(resnamelistoffset, 2);
  fs.read(nbtypes, 2);

  restypelistoffset := swap2(restypelistoffset);
  resnamelistoffset := swap2(resnamelistoffset);

  rfile^.map.nbtypes := swap2(nbtypes);
  inc(rfile^.map.nbtypes);
  rfile^.map.alltypes := AllocMem(nbtypes * sizeof(TResTypeMap));

  typetrack := rfile^.map.alltypes;


  for ctr := 0 to rfile^.map.nbtypes - 1 do
  begin
    fs.read(restype, 4);
    typetrack^.restype := string(restype);
    fs.read(nbres, 2);
    typetrack^.nbentry := swap2(nbres);
    fs.read(resoff, 2);
    resoff := swap2(resoff);

    prevpos1 := fs.Position;

    fs.seek(toppos + resmapoffset + restypelistoffset + resoff, soFromBeginning);

    inc(typetrack^.nbentry);
    typetrack^.entries := AllocMem(typetrack^.nbentry * sizeof(TResource));
    entrytrack := typetrack^.entries;


    for ctr2 := 0 to typetrack^.nbentry - 1 do
    begin

      fs.read(resid, 2);
      fs.read(resnameoffset, 2);
      entrytrack^.id := swap2(resid);
      resnameoffset := swap2(resnameoffset);

      resoffdat := 0;
      fs.read(resoffdat, 4);
      resoffdat := resoffdat and $FFFFFF00;
      fs.seek(4, soFromCurrent);
      resoffdat := swap4(resoffdat);

      prevpos2 := fs.Position;

      if resnameoffset <> $FFFF then
      begin
        fs.seek(toppos + resmapoffset + resnamelistoffset + resnameoffset, soFromBeginning);
        entrytrack^.name := readuncrypted8(fs);
      end;
      fs.Seek(toppos + resdataoffset + resoffdat, soFromBeginning);

      fs.Read(resourcesize, 4);
      resourcesize := swap4(resourcesize);
      entrytrack^.size := resourcesize;
      GetMem(entrytrack^.data, resourcesize);
      fs.Read(entrytrack^.data^, resourcesize);

      fs.Seek(prevpos2, soFromBeginning);
      inc(entrytrack);

    end;
    fs.seek(prevpos1, soFromBeginning);
    inc(typetrack);
  end;

  fs.free;

  Result := rfile;

end;

procedure CloseResFile(handle: PResFile);
var
  rfile: _PResFile;
  ctr: Integer;
  rt: PResTypeMap;
  et: PResource;
  ctr2: Integer;
begin
  if handle = nil then exit;
  rfile := handle;
  rt := rfile^.map.alltypes;
  for ctr := 0 to rfile^.map.nbtypes - 1 do
  begin
    et := rt^.entries;
    for ctr2 := 0 to rt^.nbentry - 1 do
    begin
      FreeMem(et^.data);
      inc(et);
    end;
    FreeMem(rt^.entries);
    inc(rt);
  end;
  FreeMem(rfile^.map.alltypes);
  FreeMem(rfile);
end;

function GetResource(resfile: PResFile; _type: PResType; id: integer): PResource;
var
strtype: string[4];
rfile: _PResFile;
  rt: PResTypeMap;
  et: PResource;
  ctr: Integer;
  ctr2: Integer;
begin
strtype := _type;
Result := nil;
if resfile = nil then exit;

rfile := resfile;
rt := rfile^.map.alltypes;
for ctr := 0 to rfile^.map.nbtypes - 1 do
begin
  if rt^.restype = strtype then
  begin
    et := rt^.entries;
    for ctr2 := 0 to rt^.nbentry - 1 do
    begin
      if et^.id = id then
      begin
        Result := et;
        exit;
      end;
      inc(et);
    end;
    exit;
  end;
  inc(rt);
end;

end;

end.

