unit uPieces;

interface

type
TPieceEntry = packed record
  ItemID: word;
  Left: smallint;
  Top: smallint;
end;
TPiece = packed record
  stuff: word;
  items: array [0..2] of TPieceEntry;
end;

var
currentpieces: array of TPiece;

procedure ConvertCurrentPiece;
procedure LoadPiece;
procedure UnloadPiece;
function GetPieceValue(offset: word): smallint;
procedure DrawPiece(layer: smallint = -1; celltype: smallint = -1);
procedure DrawPiecePos(layer: smallint = -1; celltype: smallint = -1);

implementation

uses uSwapper, uResources, uMacFile, Math, uImgTables;

procedure ConvertCurrentPiece;
var
  ctr: Integer;
  ctr2: Integer;
begin
  for ctr := 0 to length(currentpieces) - 1 do
  begin
    currentpieces[ctr].stuff := swap2(currentpieces[ctr].stuff);
    for ctr2 := 0 to 2 do
    begin
      currentpieces[ctr].items[ctr2].ItemID := swap2(currentpieces[ctr].items[ctr2].ItemID);
      currentpieces[ctr].items[ctr2].Left := swap2(currentpieces[ctr].items[ctr2].Left);
      currentpieces[ctr].items[ctr2].Top := swap2(currentpieces[ctr].items[ctr2].Top);
    end;
  end;

end;

procedure LoadPiece;
var
res: PResource;
begin
  UnloadPiece;
  res := PopGetResource('PIEC', 3500);
  if res <> nil then
  begin
    SetLength(currentpieces, ceil(res^.size / 20));
    move(res^.data^, currentpieces[0], res^.size);
    ConvertCurrentPiece;
  end;
  
end;

procedure UnloadPiece;
begin
  SetLength(currentpieces, 0);
end;

function GetPieceValue(offset: word): smallint;
var
idx: word;
part: word;
begin
idx := offset div 20;
part := offset mod 20;
case part of
0: Result := currentpieces[idx].stuff;
2: Result := currentpieces[idx].items[0].itemid;
4: Result := currentpieces[idx].items[0].left;
6: Result := currentpieces[idx].items[0].top;
8: Result := currentpieces[idx].items[1].itemid;
10: Result := currentpieces[idx].items[1].left;
12: Result := currentpieces[idx].items[1].top;
14: Result := currentpieces[idx].items[2].itemid;
16: Result := currentpieces[idx].items[2].left;
18: Result := currentpieces[idx].items[2].top;
end;
end;

procedure DrawPiece(layer: smallint; celltype: smallint);
begin
if celltype < 0 then
  celltype := NextBFItem^.PartFore;
if layer < 0 then
  layer := NextBFItem^.Layer;
case layer of
0: begin
  NextBFItem^.ItemID := currentpieces[celltype].items[0].ItemID;
  NextBFItem^.Left := currentpieces[celltype].items[0].Left;
  NextBFItem^.Top := currentpieces[celltype].items[0].Top;
end;
1: begin
  NextBFItem^.ItemID := currentpieces[celltype].items[2].ItemID;
  NextBFItem^.Left := currentpieces[celltype].items[2].Left;
  NextBFItem^.Top := currentpieces[celltype].items[2].Top;
end;
2,5: begin
  NextBFItem^.ItemID := currentpieces[celltype].items[1].ItemID;
  NextBFItem^.Left := currentpieces[celltype].items[1].Left;
  NextBFItem^.Top := currentpieces[celltype].items[1].Top;
end;
end;
end;

procedure DrawPiecePos(layer: smallint; celltype: smallint);
begin
if celltype < 0 then
  celltype := NextBFItem^.PartFore;
if layer < 0 then
  layer := NextBFItem^.Layer;
case layer of
0: begin
  NextBFItem^.Left := currentpieces[celltype].items[0].Left;
  NextBFItem^.Top := currentpieces[celltype].items[0].Top;
end;
1: begin
  NextBFItem^.Left := currentpieces[celltype].items[2].Left;
  NextBFItem^.Top := currentpieces[celltype].items[2].Top;
end;
2,5: begin
  NextBFItem^.Left := currentpieces[celltype].items[1].Left;
  NextBFItem^.Top := currentpieces[celltype].items[1].Top;
end;
end;


end;

end.
