unit uRect;

interface
uses uMacTypes;

function SectRect(const r1: TMacRect; const r2: TMacRect; var mix: TMacRect): boolean;
function MakeShapeRect(shape: pointer; top: smallint; left: smallint; var rect: TMacRect): TMacRect;
procedure NormalizeRect(var rect: TMacRect);
procedure MoveRect(var rect: TMacRect; top, left: smallint);
function NotSameRect(rect1: TMacRect; rect2: TMacRect): boolean;
function RectW(rect: TMacRect): smallint;
function RectH(rect: TMacRect): smallint;
procedure ComputeClipRect(var rect: TMacRect; col: smallint; row: smallint);
function EmptyRect(rect: TMacRect): boolean;
procedure UnionRect(var rect: TMacRect; rect1, rect2: TMacRect);
procedure ComputeOffsetCellRect(var rect: TMacRect; defrect: TMacRect; col, row: smallint);
procedure OffsetRect(var rect:TMacRect; x: smallint; y: smallint);
function EncloseRect(rect1, rect2: TMacRect): boolean;

implementation

uses Math, uCellUtil, uGlobals, uSwapper;

function SectRect(const r1: TMacRect; const r2: TMacRect; var mix: TMacRect): boolean;
begin
  mix.Top := max(r1.Top, r2.Top);
  mix.Left := max(r1.Left, r2.Left);
  mix.Bottom := min(r1.Bottom, r2.Bottom);
  mix.Right := min(r1.Right, r2.Right);
  Result := (mix.Top < mix.Bottom) and (mix.Left < mix.Right);
end;

function MakeShapeRect(shape: pointer; top: smallint; left: smallint; var rect: TMacRect): TMacRect;
begin
rect.Top := top;
rect.Left := left;
inc(pword(shape), 2);
rect.Right := rect.Left + swap2(pword(shape)^);
inc(pword(shape));
rect.Bottom := rect.Top + swap2(pword(shape)^);
Result := rect;
end;

procedure NormalizeRect(var rect: TMacRect);
begin
  dec(rect.Right, rect.left);
  dec(rect.Bottom, rect.Top);
  rect.Top := 0;
  rect.Left := 0;

end;


procedure MoveRect(var rect: TMacRect; top, left: smallint);
begin
  NormalizeRect(rect);
  inc(rect.Left, left);
  inc(rect.Right, left);
  inc(rect.Top, top);
  inc(rect.Bottom, top);
end;

function NotSameRect(rect1: TMacRect; rect2: TMacRect): boolean;
begin
  Result := true;
  if rect1.Top <> rect2.Top then Result := false;
  if rect1.Left <> rect2.Left then Result := false;
  if rect1.Bottom <> rect2.Bottom then Result := false;
  if rect1.Right <> rect2.Right then Result := false;
end;

function RectW(rect: TMacRect): smallint;
begin
  Result := rect.Right - rect.Left;
end;
function RectH(rect: TMacRect): smallint;
begin
  Result := rect.Bottom - rect.Top;
end;

procedure ComputeClipRect(var rect: TMacRect; col: smallint; row: smallint);
var
temprect: TMacRect;
begin
temprect.Left := CELL_LEFT[col];
temprect.Right := temprect.left + 51;
temprect.Bottom := CELL_BOTTOM[row];
temprect.Top := temprect.Bottom - 120;
SectRect(temprect, backbuffer.Rect, rect);
if customized and (row = 0) then
dec(rect.Top, 5);
end;

function EmptyRect(rect: TMacRect): boolean;
begin
  Result := (rect.top = 0) and (rect.left = 0) and (rect.bottom = 0) and (rect.right = 0);  
end;

procedure UnionRect(var rect: TMacRect; rect1, rect2: TMacRect);
begin
  rect.top := min(rect1.top, rect2.top);
  rect.left := min(rect1.left, rect2.left);
  rect.bottom := max(rect1.bottom, rect2.bottom);
  rect.right := max(rect1.right, rect2.right);

end;

procedure ComputeOffsetCellRect(var rect: TMacRect; defrect: TMacRect; col, row: smallint);
begin
  if (col <> 30) and (row <> 30) then
  begin
    rect := defrect;
    OffsetRect(rect,col*51,row*120);
    if not SectRect(rect, BackBuffer.Rect, rect) then
    begin
      rect.top := 0;
      rect.left := 0;
    end;
  end
  else
  begin
    rect.top := 0;
    rect.left := 0;
  end;
end;

procedure OffsetRect(var rect:TMacRect; x: smallint; y: smallint);
begin
  inc(rect.top, y);
  inc(rect.bottom, y);
  inc(rect.left, x);
  inc(rect.right, x);
end;

function EncloseRect(rect1, rect2: TMacRect): boolean;
var
rect: TMacRect;
begin
UnionRect(rect, rect1, rect2);
Result := (rect1.top = rect.top) and (rect1.left = rect.left) and (rect1.bottom = rect.bottom) and (rect1.right = rect.right);
end;

end.
