unit uRoomUtil;

interface

uses uPop2Types, uMacTypes;

procedure CalcBlue(room: word);
procedure MapSurroundingRooms;
function IsCustomizedRoom(room: word): boolean;
procedure InitState(cell: word);
function FirstInRow(row: smallint):smallint;
function GetObject(room:word;col:smallint;row:smallint;var back: TBackPair; default: word):word;
procedure ComputeImmediate;
function CellToCol(room: word; cell: word): smallint;
function CellToRow(room: word; cell: word): smallint;
procedure GetAboveAry;
procedure GetBelowAry;
procedure GetOS(cell:smallint);
procedure SelectObjects;
procedure ClearAboveCustomized;
function GetHorsePtInPlane(room: word; layer: word): PMacPoint;
function VisibleAbovePiece(col: smallint; row: smallint): boolean;
function PieceAboveId(col, row: smallint): word;
function PieceAboveRIghtId(col, row: smallint): word;
function PieceRIghtId(col, row: smallint): word;
function PieceLeftId(col, row: smallint): word;
function PieceBelowId(col, row: smallint): word;

function GetRoomId(room: word; row, col: smallint; default: word): word;


implementation

uses uGlobals, uCellUtil, uRect;

const
silverhorse: TMacPoint = (x:$FC; y:$28);
redhorse: TMacPoint = (x:$4A; y:$C6);
goldenhorse: TMacPoint = (x:$8D; y:$4E);


procedure CalcBlue(room: word);
begin
  if room = 0 then exit;
  CurrentForeTable := @CurrentLevel.ForeTables[room - 1];
  CurrentBackTable := @CurrentLevel.BackTables[room - 1];

end;

procedure MapSurroundingRooms;
var
  ctr: Integer;
  a: array[0..31] of array[0..29] of byte;
  ctr2: Integer;
begin
  bottomrightroom := 0;
  bottomleftroom := 0;
  toprightroom := 0;
  topleftroom := 0;

  for ctr2 := 0 to currentlevel.maxrooms - 1 do
    for ctr := 0 to 29 do
      a[ctr2][ctr] := currentlevel.backtables[9][ctr].Back2 and $3F;
  a[currentlevel.maxrooms][0] := $FF;

  if currentroom <> 0 then
  begin
    CalcBlue(currentroom);
    with currentlevel.Relations[currentroom - 1] do
    begin
      LeftRoom := Left;
      RightRoom := Right;
      TopRoom := Up;
      BottomRoom := Down;
    end;
    if TopRoom <> 0 then
    begin
      TopLeftRoom := currentlevel.Relations[toproom - 1].Left;
      TopRightRoom := currentlevel.Relations[toproom - 1].Right;
    end
    else
    begin
      if LeftRoom <> 0 then
        TopLeftRoom := currentlevel.Relations[leftroom - 1].Up;
      if RightRoom <> 0 then
        TopRightRoom := currentlevel.Relations[rightroom - 1].Up;
    end;

    if BottomRoom <> 0 then
    begin
      BottomLeftRoom := currentlevel.Relations[bottomroom - 1].Left;
      BottomRightRoom := currentlevel.Relations[bottomroom - 1].Right;
    end
    else
    begin
      if LeftRoom <> 0 then
        BottomLeftRoom := currentlevel.Relations[leftroom - 1].Down;
      if RightRoom <> 0 then
        BottomRightRoom := currentlevel.Relations[rightroom - 1].Down;
    end;
    customized := IsCustomizedRoom(currentroom);
  end
  else
  begin
    bottomroom := 0;
    toproom := 0;
    rightroom := 0;
    leftroom := 0;
  end;
  
end;

function IsCustomizedRoom(room: word): boolean;
var
btable: PBackEntry;
  ctr: Integer;
begin
  Result := false;
  btable := @currentlevel.BackTables[room - 1];
  for ctr := 0 to 29 do
  begin
    if (btable^[ctr].Back1 and $C000) <> 0 then
    begin
      Result := true;
      exit;
    end;
  end;
end;

procedure InitState(cell: word);
begin
  if currentforetable^[cell] = CELL_TUNNEL then
    currentbacktable^[cell].Back1 := (currentbacktable^[cell].Back1 and $FF7F);
end;

function FirstInRow(row: smallint):smallint;
begin
case row of
-4: Result := -31;
-3: Result := -21;
-2: Result := -11;
-1: Result := -1;
0: Result := 0;
1: Result := 10;
2: Result := 20;
3: Result := 30;
end;
end;

function GetObject(room:word;col:smallint;row:smallint;var back: TBackPair; default: word):word;
var
cell: smallint;
begin
  Result := default;
  if row = -1 then
    row := 2;
  if col = -1 then
  begin
    Result := immediateleft[row].fore;
    back := immediateleft[row].back;
  end
  else if col = 10 then
  begin
    Result := immediateright[row].fore;
    back := immediateright[row].back;
  end
  else if room <> 0 then
  begin
    cell := FirstInRow(row)+col;
    Result := currentforetable[cell];
    back := currentbacktable[cell];
  end
  else if currentlevel.leveltype=3 then
  begin
    back.Back1 := 0;
    back.Back2 := 0;
  end
  else if currentlevel.leveltype=2 then
  begin
    back.Back1 := 0;
    back.Back2 := 9;
  end
  else if currentlevel.leveltype in [4,5] then
  begin
    back.Back1 := 0;
    back.Back2 := 0;
  end
  else
  begin
    back.Back1 := 0;
    back.Back2 := ((col and 1) shl 8) + 9;
  end;


end;

procedure ComputeImmediate;
var
default: word;
  ctr: Integer;
begin
  if (currentroom = 0) then
  begin
    if (currentlevel.leveltype = 4) and (currentlevel.levelnumber=7) then
    begin
      if bottomroom in [7,15,32] then
        default := 20
      else
        default := 0;
    end
    else
      default := 0;
  end
  else
    default := 20;
  CalcBlue(leftroom);
  for ctr := 0 to 2 do
  begin
    immediateleft[ctr].fore := GetObject(leftroom,9,ctr,immediateleft[ctr].back, default);
    if (custompieces <> nil) and ((custompieces^.ID <> $1F) or (ctr = 0)) then
      immediateleft[ctr].Back.Back1 := immediateleft[ctr].Back.Back1 or $C000
    else
      immediateleft[ctr].Back.Back1 := immediateleft[ctr].Back.Back1 and $3FFF
  end;
  CalcBlue(rightroom);
  for ctr := 0 to 2 do
    immediateright[ctr].fore := GetObject(rightroom,0,ctr,immediateright[ctr].back, default);
  CalcBlue(currentroom);
end;

function CellToCol(room: word; cell: word): smallint;
begin
Result := cell mod 10;
if not (room in [currentroom, toproom, bottomroom]) then
begin
  if (Result = 9) and (room in [topleftroom, leftroom, bottomleftroom]) then
    Result := -1
  else if (Result = 0) and (room in [toprightroom, rightroom, bottomrightroom]) then
    Result := 11
  else
    Result := 30;
end;
end;

function CellToRow(room: word; cell: word): smallint;
begin
Result := cell div 10;
if not (room in [currentroom, leftroom, rightroom]) then
begin
  if (Result = 2) and (room = toproom) then
    Result := -1
  else if (Result = 0) and (room = bottomroom) then
    Result := 3
  else
    Result := 30;
end;
end;

procedure GetAboveAry;
var
defaulttop: word;
oldbottom: word;
current: word;
left: word;
row: smallint;
  ctr: Integer;
begin
  if drawingrow = -1 then exit;
  oldbottom := bottomroom;
  if drawingrow = 0 then
  begin
    current := toproom;
    left := topleftroom;
    bottomroom := currentroom;
  end
  else
  begin
    current := currentroom;
    left := leftroom;
  end;
  row := drawingrow - 1;
  if customized or (currentlevel.leveltype in [4,5]) then
    defaulttop := CELL_EMPTY
  else
    defaulttop := CELL_FLOOR;
  CalcBlue(left);
  above[0].fore := GetObject(left, 9, row, above[0].Back, defaulttop);
  if (custompieces <> nil) and (drawingrow = 0) then
    above[0].back.Back1 := above[0].back.Back1 or $C000;
  CalcBlue(current);
  for ctr := 1 to 10 do
  begin
    above[ctr].fore := GetObject(current, ctr-1, row, above[ctr].Back, defaulttop);
    if customized and (drawingrow=0) and (currentlevel.leveltype=3) and IsFloor(currentlevel.foretables[current-1][ctr-1]) then
      above[ctr].Back.Back1 := (above[ctr].Back.Back1 and $3FFF) or $4000;
  end;
  CalcBlue(currentroom);
  bottomroom := oldbottom;
end;

procedure GetBelowAry;
var
defaulttop: word;
current: word;
left: word;
row: smallint;
  ctr: Integer;
begin
  if drawingrow = 2 then
  begin
    current := bottomroom;
    left := bottomleftroom;
    row := 0;
  end
  else
  begin
    current := currentroom;
    left := leftroom;
    row := drawingrow + 1;
  end;

  CalcBlue(left);

  if current = 0 then
    defaulttop := CELL_EMPTY
  else
    defaulttop := CELL_WALL;

  below[0].fore := GetObject(left, 9, row, below[0].Back, defaulttop);

  CalcBlue(current);
  for ctr := 1 to 10 do
    below[ctr].fore := GetObject(current, ctr-1, row, below[ctr].Back, defaulttop);
  CalcBlue(currentroom);
end;

procedure GetOS(cell:smallint);
begin
  os_fore := currentforetable[cell];
  os_back := currentbacktable[cell];
end;

procedure SelectObjects;
var
defaultcell: word;
begin
if ((drawingrow=2) and (not drawingfull)) or (drawingrow=-1) then
begin
  if currentlevel.leveltype = 4 then
  begin
    if (currentlevel.LevelNumber = 7) and (bottomroom in [7,15,32]) then
      defaultcell := CELL_FLOOR
    else
      defaultcell := CELL_EMPTY;
  end
  else if currentlevel.leveltype = 5 then
    defaultcell := CELL_EMPTY
  else
    defaultcell := CELL_FLOOR;
end
else
  defaultcell := CELL_WALL;

if not drawingfull then
  ComputeClipRect(DrawingRect, drawingcol, drawingrow)
else
  DrawingRect := backbuffer.Rect;

OS_Fore := GetObject(currentroom, drawingcol, drawingrow, OS_Back, defaultcell);
OS_LeftFore := GetObject(currentroom, drawingcol-1, drawingrow, OS_LeftBack, defaultcell);
OS_RightFore := GetObject(currentroom, drawingcol+1, drawingrow, OS_RightBack, defaultcell);

if (drawingrow = -1) and (CustomPieces <> nil) then
begin
  OS_Back.Back1 := OS_Back.Back1 or $C000;
  OS_LeftBack.Back1 := OS_LeftBack.Back1 or $C000;
  OS_RightBack.Back1 := OS_RightBack.Back1 or $C000;
end;


end;

procedure ClearAboveCustomized;
var
  ctr: Integer;
begin
  for ctr := 0 to 10 do
  begin
    Above[ctr].Fore := 0;
    Above[ctr].Back.Back1 := $C000;
    Above[ctr].Back.Back2 := 0;
  end;
end;

function GetHorsePtInPlane(room: word; layer: word): PMacPoint;
begin
  Result := nil;
  if (layer = 0) and (room = 2) and (currentlevel.LevelNumber = 9) then
    result := @silverhorse
  else if (layer = 1) and (room = 1) and (currentlevel.LevelNumber = 14) then
    result := @goldenhorse
  else if (layer = 1) and (room = 22) and (currentlevel.LevelNumber = 10) then
    result := @redhorse;
end;

function VisibleAbovePiece(col: smallint; row: smallint): boolean;
var
pieceid: word;
begin
  Result := false;
  pieceid := PieceAboveId(col, row);
  if IsSpace(pieceid) or (pieceid in [CELL_RAISED, CELL_LOWERED]) or ((currentlevel.LevelNumber = 13) and (pieceid = CELL_PILLAR)) then
    Result := true
  else
  begin
    pieceid := PieceAboveRightId(col, row);
    if IsSpace(pieceid) or (pieceid in [CELL_RAISED, CELL_LOWERED]) or ((currentlevel.LevelNumber = 13) and (pieceid = CELL_PILLAR)) then
      Result := true;
  end;
end;

function PieceAboveId(col, row: smallint): word;
var
room: word;
begin
  if row > 0 then
  begin
    dec(row);
    room := currentroom;
  end
  else
  begin
    room := toproom;
    row := 2;
  end;
  if col<0 then
  begin
    if room<>0 then
      room := currentlevel.Relations[room-1].Left;
    inc(col, 10);
  end
  else if col >= 10 then
  begin
    if room <> 0 then
      room := currentlevel.Relations[room-1].Right;
    dec(col, 10);
  end;
  Result := GetRoomID(room, row, col, 1);
end;

function PieceAboveRIghtId(col, row: smallint): word;
var
room: word;
begin
  if row > 0 then
  begin
    dec(row);
    room := currentroom;
  end
  else
  begin
    room := toproom;
    row := 2;
  end;
  if room <> 0 then
  begin
    if col < -1 then
    begin
      if room<>0 then
        room := currentlevel.Relations[room-1].Left;
      inc(col, 9);

    end
    else if col >= 9 then
    begin
      if room <> 0 then
        room := currentlevel.Relations[room-1].Right;
      dec(col, 10);
    end
    else
      inc(col, 1);
    Result := GetRoomID(room, row, col, 1);
  end;

end;

function PieceRIghtId(col, row: smallint): word;
var
room: word;
cell: smallint;
begin
 if col <> 9 then
 begin
   if (row = 0) and (drawingrow = -1) then
     room := bottomroom
   else
   begin
     room := currentroom;
     if (row = -1) and (drawingrow = -1) then
       row := 2;
   end;
   inc(col);
 end
 else
 begin
   if (row = -1) and (drawingrow = -1) then
     room := toprightroom
   else
     room := rightroom;
   col := 0;
 end;
 if row = 3 then
 begin
   if room = rightroom then
     room := bottomrightroom
   else
     room := bottomroom;
   row := 0;
 end;
 if room <> 0 then
 begin
   cell := FirstInRow(row) + col;
   Result := currentlevel.ForeTables[room-1][cell];
 end
 else
 begin
   if (currentlevel.LevelType = 4) and (currentlevel.levelnumber = 7) and not (currentroom in [7,15,32]) then
     Result := CELL_EMPTY
   else
     Result := CELL_WALL;
   
 end;
end;

function PieceLeftId(col, row: smallint): word;
var
room: word;
begin
  if col <> 0 then
  begin
    if (row = 0) and (drawingrow = -1)  then
      room := bottomroom
    else
      room := currentroom;
    dec(col,1);
  end
  else
  begin
    if (row = -1) and (drawingrow = -1) then
      room := topleftroom
    else
      room := leftroom;
    col := 9;
  end;
  Result := GetRoomID(room, row, col, 20);
end;

function PieceBelowId(col, row: smallint): word;
var
room: word;
begin
  if row < 2 then
  begin
    if (row = 0) and (drawingrow = -1) then
      room := bottomroom
    else
      room := currentroom;
    inc(row, 1);
  end
  else
  begin
    row := 0;
    room := bottomroom;
  end;
  if col < 0 then
  begin
    room := currentlevel.relations[room - 1].Left;
    inc(col, 10);
  end
  else if col >= 10 then
  begin
    room := currentlevel.relations[room - 1].Right;
    dec(col, 10);
  end;
  Result := GetRoomID(room, row, col, 20);
end;


function GetRoomId(room: word; row, col: smallint; default: word): word;
var
cell: smallint;
begin
  Result := default;
  if room = 0 then exit;
  //if room > currentlevel.MaxRooms then exit;
  cell := FirstInRow(row)+col;
  if cell < 0 then exit;
  if cell >= 30 then exit;
  Result := currentlevel.ForeTables[room-1][cell];
end;

end.
