unit uShapeSets;

interface

uses uMacTypes;

function GetBGSetShapeItem(id: integer): pointer;
function GetBGSetImgSize(id: integer; var size: TMacPoint): boolean;
procedure LoadSet(index: word; id: smallint);
procedure LoadBGSet(updatepalette: boolean);
procedure LoadFLaskSet;
procedure LoadFlameShapes(flag: word);
function GetHorseShape: pointer;
function GetCarpetShape(index: word): pointer;
procedure UnloadSet(index: word);
procedure UnloadFlameShapes;
procedure LoadKidSet;
procedure LoadOppSet(opptype: smallint);
procedure GetSwordType;
procedure LoadSwordSet(swtype: smallint);
procedure LoadKidAttachFrames(swtype: smallint);


implementation

uses uGlobals, uMacFile, uResources, uShapes, uShapeList, uColorTables, uPop2Types, uFrames,
uSwapper;

function GetBGSetShapeItem(id: integer): pointer;
var
idx: integer;
curfile: PResFile;
begin
idx := id - 1;
Result := shapesets[4]^.Content[idx].shape;
if Result = nil then
begin
  curfile := POPCurResFile;
  PoPUseResFile(levelresources[currentlevel.LevelType]);
  Result := GetShape(3500 + id);
   shapesets[4]^.Content[idx].shape := Result;
  PopUseResFile(curfile);
end;

end;

procedure LoadSet(index: word; id: smallint);
begin
  if shapesets[index] = nil then
    shapesets[index] := LoadShapeList(id, false);
end;

procedure LoadBGSet(updatepalette: boolean);
var
cur: PResFile;
begin
cur := PoPCurResFile;
PopUseResFile(levelresources[currentlevel.leveltype]);
LoadSet(4, 3500);
if updatepalette then
  PoPSetPalette(3500, 64);
PoPUseResFile(cur);
if currentlevel.leveltype = 6 then
  LoadFlameShapes(2);



end;

procedure LoadFlameShapes(flag: word);
var
first: word;
last: word;
cur: PResFile;
  ctr: Word;
begin
  if length(flameshapes) > 0 then exit;
  cur := PoPCurResFile;
  PopUseResFile(resKid);

  setlength(flameshapes, 46);
  if (flag and 1) <> 0 then
    first := 0
  else
    first := 15;
  if (flag and 2) <> 0 then
    last := 45
  else
    last := 33;

  for ctr := first to last do
    flameshapes[ctr] := GetShape(ctr + 4077);

  PopUseResFile(cur);
end;

function GetHorseShape: pointer;
var
curfile: PResFile;
begin
  if horseshape = nil then
  begin
    curfile := PopCurResFile;
    PopUseResFile(resPrince);
    horseshape := GetShape(25360);
    PopUseResFile(curfile);
  end;
  Result := horseshape;
end;

function GetCarpetShape(index: word): pointer;
var
curfile: PResource;
begin
  if carpetshapes[index] = nil then
  begin
  curfile := PoPCurResFile;
  PopUseResFile(levelresources[currentlevel.leveltype]);
  carpetshapes[index] := GetShape(index + 25303);
  PopUseResFile(curfile);
  end;
  Result := carpetshapes[index];
end;

procedure UnloadSet(index: word);
begin
  if (index < 5) and (shapesets[index] <> nil) then
  begin
    UnloadShapeList(shapesets[index]);
    shapesets[index] := nil;
  end;

end;

procedure UnloadFlameShapes;
var
  ctr: Integer;
begin
  if length(flameshapes) = 0 then exit;
  for ctr := 0 to length(flameshapes) - 1 do
  FreeMem(flameshapes[ctr]);
  SetLength(flameshapes, 0);
end;

procedure LoadFlaskSet;
var
curfile: PResFile;
begin
  if currentlevel.HasFlask then
  begin
  curfile := PoPCurResFile;
  PopUseResFile(resPrince);
  LoadSet(1, 3000);

  if currentlevel.leveltype <> 6 then
    PopSetPalette(3000, 224);

  PopUseResFile(curfile);
  end
  else
  UnloadSet(1);

end;

procedure LoadKidSet;
var
curfile: PResFile;
frames: PResource;
begin
  curfile := PoPCurResFile;
  PopUseResFile(resKid);
  LoadSet(2, 25001);
  frames := PopGetResource('FRAM',25001);
  SetLength(kidframes, frames^.size div sizeof(TFrameEntry));
  move(frames^.Data^, kidframes[0], frames^.size);
  ConvertKidFrames;
  //ConvertAFrames(0);
  PopUseResFile(curfile);

end;

procedure LoadOppSet(opptype: smallint);
var
frames: PResource;
curfile: PResFile;
r: integer;
begin
  if (opptype = -1) or (currentopptype<>opptype) or (shapesets[3]=nil) then
  begin
    UnloadSet(3);
    SetLength(OppFrames, 0);
    SetLength(OppAFrames, 0);
    SetLength(headmoves, 0);
    if (currentopptype <> -1) and (opptype<>currentopptype) then
      CloseResFile(resOpp);
  end;
  if opptype <> -1 then
  begin
    curfile := PopCurResFile;
    resOpp := OpenResFile(basepath + opppath[opptype]);
    PopUseResFile(resOpp);
    LoadSet(3, 750);
    PopSetPalette(750, 32);
    frames := PopGetResource('FRAM',750);
    SetLength(oppFrames, frames^.size div sizeof(TFrameEntry));
    move(frames^.Data^, oppframes[0], frames^.size);
    ConvertOppFrames;
    frames := PopGetResource('AFRM',750);
    SetLength(oppaframes, frames^.size div sizeof(TAFrameEntry));
    move(frames^.Data^, oppaframes[0], frames^.size);
    ConvertOppAFrames;
    if opptype in [5,6] then
    begin
    r := 0;
    frames := PopGetResource(prestype(r),755);
    SetLength(headmoves, frames^.size div sizeof(THeadMove));
    move(frames^.Data^, headmoves[0], frames^.size);
    ConvertHeadMoves;
    end;
    PopUseResFile(curfile);
  end;
  currentopptype := opptype;

end;

function GetBGSetImgSize(id: integer; var size: TMacPoint): boolean;
var
shape: pointer;
begin
shape := GetBGSetShapeItem(id);
if shape <> nil then
begin
size.y := swap2(pword(integer(shape) + 6)^);
size.x := swap2(pword(integer(shape) + 4)^);
end
else
begin
  size.x := 0;
  size.y := 0;
end;
Result := (size.x > 0) and (size.y > 0);
end;

procedure GetSwordType;
begin
  if currentlevel.levelnumber = 6 then
    SwordType := -1
  else if currentlevel.LevelNumber in [7,8] then
    SwordType := 2
  else
    SwordType := 1;

end;

procedure LoadSwordSet(swtype: smallint);
var
curfile: PResFile;
begin
if currentlevel.leveltype <> 1 then
begin
  if swtype <> currentkidafrm then
  begin
    if length(KidAFrames)<>0 then
    SetLength(KidAFrames, 0);
    UnloadSet(0);
  end;
  if length(KidAFrames) = 0 then
  LoadKidAttachFrames(swtype);
  curfile := PoPCurResFile;
  PopUseResFIle(resPrince);
  if swtype = 2 then
  begin
    LoadSet(0,1200);
  end
  else
  begin
    LoadSet(0,1000);
  end;

  if (currentroom <> 1) or (currentlevel.levelnumber <> 14) then
  begin
    PopSetPalette(1000+currentlevel.leveltype, $F0);
  end;

  PopUseResFile(curfile);

  currentkidafrm := swtype;
end
else
begin
if length(KidAFrames) = 0 then
  LoadKidAttachFrames(swtype);

end;
end;

procedure LoadKidAttachFrames(swtype: smallint);
var
frames: PResource;
begin
  if swtype = 2 then
    frames := PopGetResource('AFRM',25201)
  else
    frames := PopGetResource('AFRM',25001);
  SetLength(kidaframes, frames^.size div sizeof(TAFrameEntry));
  move(frames^.Data^, kidaframes[0], frames^.size);
  ConvertKidAFrames;
  //ConvertAFrames(0);

end;

end.
